#include "Driver.h"
#include "Disk.h"
#include "ImageIO.h"
#include "ImageProcessing.h"

void Driver::execGenericOp()
{
	ENSURE(this->params.genericParams.intList1.size() >= 1);
	int opCode = this->params.genericParams.intList1[0];

	switch(opCode)
	{
	case 0:
		scaleAndRenameImgs();
		break;

	default:
		REPORT_FAILURE("Unknown operation code: %i\n", opCode);
		break;
	}
}

void Driver::scaleAndRenameImgs()
{
	ENSURE(this->params.genericParams.floatList.size() == 1);
	float scale = this->params.genericParams.floatList[0];
	float blurSig = 1.0f;
	if(scale < 1.0f)
	{
		float scalePow = -(log(scale) / log(2.0f));
		blurSig *= pow(2.0f, scalePow - 1.0f);
	}

	char strBuf[1024];

	REPORT("Scaling and renaming files...\n");

	for(uint imgIndex = 0; imgIndex < this->params.imgFNs.size(); imgIndex++)
	{
		string inFN = this->params.imgFNs[imgIndex];
		string baseDir = Disk::GetBaseDirName(inFN);
		string imgExt = Disk::GetFileNameExt(inFN);

		sprintf(strBuf, "vid-%03i", imgIndex);
		string outID = strBuf;

		string outFN = baseDir + outID + "." + imgExt;
		INSIST(outFN.compare(inFN) != 0);

		CFloatImage inImg;
		ImageIO::ReadFile(inImg, inFN);

		CFloatImage outImg;
		ImageProcessing::ScaleImage(outImg, inImg, scale, true, blurSig);

		ImageIO::WriteFile(outImg, outFN);

		printf("%s -> %s\n", inFN.c_str(), outFN.c_str());
	}	

	REPORT("\tDone.\n");
}

