#include "Driver.h"
#include "Filter_Driver.h"

Driver::Driver(OptFileParser::OptFileParams driverParams)  
{
	this->params = driverParams;
}

void Driver::Start()
{
	srand(this->params.seed); 

	switch(this->params.appOP)
	{
	case OptFileParser::OP_FILTER:
		execFilterOp();
		break;

	case OptFileParser::OP_FILTER_VIDEO:
	case OptFileParser::OP_FILTER_VIDEO_CR:
		execFilterVideoOp();
		break;

	case OptFileParser::OP_TEST_FILTER:
		execTestFilterOp();
		break;

	case OptFileParser::OP_GENERIC:
		execGenericOp();
		break;

	default:
		REPORT_FAILURE("Unknown app operation - %i\n", this->params.appOP);
		break;
	}
}


void Driver::execFilterOp()
{
	INSIST(this->params.imgFNs.size() > 0);
	for(vector<string>::const_iterator imgFN = this->params.imgFNs.begin(); 
		imgFN != this->params.imgFNs.end(); 
		imgFN++)
	{
		Filter_Driver::Filter(*imgFN, this->params.fltParams);
	}
}


void Driver::execFilterVideoOp()
{
	INSIST(this->params.imgFNs.size() > 0);

	this->params.gsvsParams.imgFNs = this->params.imgFNs;
	GS_VSolver gsvSolver(this->params.gsvsParams);

	switch(this->params.appOP)
	{
		case OptFileParser::OP_FILTER_VIDEO:
			if(this->params.gsvsParams.inCoreFrameLimit < 0)
				gsvSolver.Solve();
			else
				GS_VSolver::SolveOutOfCore(this->params.gsvsParams);
			break;

		case OptFileParser::OP_FILTER_VIDEO_CR:
			gsvSolver.CombineResults();
			break;

		default:
			REPORT_FAILURE("Unknown app operation - %i\n", this->params.appOP);
			break;
	}
}

void Driver::execTestFilterOp()
{
	INSIST(this->params.imgFNs.size() > 0);
	for(vector<string>::const_iterator imgFN = this->params.imgFNs.begin(); 
		imgFN != this->params.imgFNs.end(); 
		imgFN++)
	{
		Filter_Driver::TestFilter(*imgFN, this->params.fltParams);
	}
}

