#include "OptFileParser.h"

OptFileParser::OptFileParams OptFileParser::OptFileParams::Default()
{
	OptFileParser::OptFileParams params;

	params.seed          = (int) time(NULL);
	params.appOP         = OptFileParser::OP_DEFAULT;
	params.imgRange      = IntPair(-1, -1);
	params.outDir        = "." + OS_PATH_DELIMIT_STR;
	params.inDir         = "." + OS_PATH_DELIMIT_STR;
	params.useDefaultFNs = false;

	params.fltParams     = Filter_Driver::Params::Default();
	params.gspParams     = GS_Problem::Params::Default();
	params.gsvsParams    = GS_VSolver::Params::Default();
	
	return params;
}

void OptFileParser::OptFileParams::Dump()
{
	printf("Global Options:\n");
	printf("  Seed:   %u\n", seed);
	printf("  outDir: %s\n", this->outDir.c_str());
	printf("  inDir:  %s\n", this->inDir.c_str());

	INSIST(this->imgRange.second >= this->imgRange.first);
	if(useDefaultFNs == false)
	{
		//bug
		//INSIST(this->imgFNs.size() > 0);
		IntPair range = this->imgRange;
		if((range.first == -1) && (range.second == -1))
		{
			range.first  = 0;
			range.second = (int) this->imgFNs.size() - 1;
		}
		CHECK_RANGE(range.first,  0, (int) this->imgFNs.size() - 1);
		CHECK_RANGE(range.second, 0, (int) this->imgFNs.size() - 1);

		printf("  Images: \n");
		for(int iImg = range.first; iImg <= range.second; iImg++)
		{
			printf("    %s\n", this->imgFNs[iImg].c_str());
		}
	}
	else
	{	
		INSIST(this->imgFNs.size() == 0);
		INSIST(this->imgRange.first >= 0);
		printf("  imgRange: <%i,%i>\n", imgRange.first, imgRange.second);
	}
			
	printf("  App OP: ");
	switch(appOP)
	{		
	case OP_FILTER:
		printf("OP_FILTER\n");
		this->fltParams.Dump("  ");
		break;

	case OP_FILTER_VIDEO:
		printf("OP_FILTER_VIDEO\n");
		this->gsvsParams.Dump("  ");
		break;

	case OP_FILTER_VIDEO_CR:
		printf("OP_FILTER_VIDEO_CR\n");
		this->gsvsParams.Dump("  ");
		break;

	case OP_TEST_FILTER:
		printf("OP_TEST_FILTER\n");
		printf("    filter type:  %s\n", Filter_Driver::Params::GetFilterName(this->fltParams.type).c_str());
		break;

	case OP_GENERIC:
		printf("OP_GENERIC\n");
		this->genericParams.Dump("  ");
		break;

	default:
		REPORT_FAILURE("Unknown app operation - %i\n", appOP);
		break;
	}	
	
	printf("\n");
	fflush(stdout);
}

