#ifndef __OPT_FILE_PARSER_H__
#define __OPT_FILE_PARSER_H__

#include <time.h>
#include "Definitions.h"
#include "GenericOpts.h"
#include "Filter_Driver.h"
#include "Curve.h"

class OptFileParser
{
public:

	typedef enum
	{		
		OP_FILTER = 1,
		OP_FILTER_VIDEO,
		OP_FILTER_VIDEO_CR,
		OP_TEST_FILTER,
		OP_GENERIC,
		OP_DEFAULT = OP_FILTER
	} AppOperation;

	typedef struct _OptFileParams
	{
		int seed;
		AppOperation appOP;
		IntPair imgRange;
		vector<string> imgFNs;
		string outDir, inDir;
		bool useDefaultFNs;

		GS_Problem::Params    gspParams;
		GS_VSolver::Params    gsvsParams;
		Filter_Driver::Params fltParams;		
		GenericOpts           genericParams;		
		
		static _OptFileParams Default();
		void Dump();
	} OptFileParams;
 
	OptFileParams Load(const char *optionsFN, char* argv[], bool dumpOptionsRead = true);

private:

	bool readIndex(     string option, Indices                      &indices,      string optStr, vector<string> &optVals);
	bool readImgShape(  string option, CShape                       &imgShape,     string optStr, vector<string> &optVals);
	bool readProbShape( string option, GS_P_Shape                   &probShape,    string optStr, vector<string> &optVals);
	bool readLevelsType(string option, GS_ProblemLevels::LevelsType &levelsType,   string optStr, vector<string> &optVals);
	bool readEnum(      string option, int                          &enumVal,      string optStr, vector<string> &optVals, const char** enumStrs);
	bool readFilterType(string option, Filter_Driver::FT_Type       &fltType,      string optStr, vector<string> &optVals);
	bool readAppOp(		string option, AppOperation                 &appOP,        string optStr, vector<string> &optVals);
	bool readBool(      string option, bool                         &boolTarget,   string optStr, vector<string> &optVals);
	bool readInt(       string option, int                          &intTarget,    string optStr, vector<string> &optVals);
	bool readUint(      string option, uint                         &uintTarget,   string optStr, vector<string> &optVals);
	bool readFloat(     string option, float                        &floatTarget,  string optStr, vector<string> &optVals);
	bool readDouble(    string option, double                       &doubleTarget, string optStr, vector<string> &optVals);
	bool addString(     string option, vector<string>               &strArray,     string optStr, vector<string> &optVals);
	bool addInt(        string option, vector<int>                  &intArray,     string optStr, vector<string> &optVals);
	bool addFloat(      string option, vector<float>                &floatArray,   string optStr, vector<string> &optVals);
	bool addBool(       string option, vector<bool>                 &boolArray,    string optStr, vector<string> &optVals);
	bool addDirName(    string option, vector<string>               &strArray,     string optStr, vector<string> &optVals);
	bool addProbFilter( string option, vector<GS_Problem::Filter>   &filters,      string optStr, vector<string> &optVals);
	bool readString(    string option, string                       &strTarget,    string optStr, vector<string> &optVals);
	bool readDirName(   string option, string                       &strTarget,    string optStr, vector<string> &optVals);
	bool readCurve(     string option, Curve                        &curve,        string optStr, vector<string> &optVals);
		
	void neutralizePaths(string &line);

	void fixImgFNs(OptFileParams& params);
	void fixOutDirs(OptFileParams& params);
	void fixVS_Params(OptFileParams& params);

	bool readFltParams(          Filter_Driver::Params&   params, string optStr, vector<string> lineTokens);	
	bool readGSVS_Params(        GS_VSolver::Params&      params, string optStr, vector<string> lineTokens);	
	bool readGSP_Params(         GS_Problem::Params&      params, string optStr, vector<string> lineTokens);	
	bool readGSS_Params(         GS_Solver::Params&       params, string optStr, vector<string> lineTokens);	
	bool readFltSharpenParams(   GSF_Sharpen::Params&     params, string optStr, vector<string> lineTokens);	
	bool readFltEleParams(       GSF_ELE::Params&         params, string optStr, vector<string> lineTokens);	
	bool readFltMotionBlurParams(GSF_MotionBlur::Params&  params, string optStr, vector<string> lineTokens);	
	bool readFltNprParams(       GSF_NPR::Params&         params, string optStr, vector<string> lineTokens);	
	bool readFltGradCurveParams( GSF_GradCurve::Params&   params, string optStr, vector<string> lineTokens);	
	bool readFltToneMapParams(   GSF_ToneMap::Params&     params, string optStr, vector<string> lineTokens);	
	bool readFltColorizeParams(  GSF_Colorize::Params&    params, string optStr, vector<string> lineTokens);
	bool readFltTestParams(      GSF_Test::Params&        params, string optStr, vector<string> lineTokens);
	bool readFltRelightParams(   GSF_Relight::Params&     params, string optStr, vector<string> lineTokens);
	bool readFltDeblockParams(   GSF_Deblock::Params&     params, string optStr, vector<string> lineTokens);
	bool readGenericParams(      GenericOpts&             params, string optStr, vector<string> lineTokens);
};

#endif

