#include "GS_Problem.h"
#include "ImageIO.h"
#include "Disk.h"

string GS_Problem::GetResultImgFN(string id, string fileExt) const
{
	return GS_Problem::getFN(id, "result", fileExt);
}

string GS_Problem::getFN(string id, string suffix, string fileExt)
{
	string baseDir = Disk::GetBaseDirName(id);
	string baseID = Disk::GetBaseFileName(id);
	return baseDir + suffix + "-" + baseID + "." + fileExt;
}

void GS_Problem::Write(string id, string fileExt, Indices levelsToUse)  
{
	this->input.Write(id, fileExt);
	WriteSol(id, fileExt);
	this->problemLevels.Write(id, fileExt, levelsToUse);
	this->params.Write(id);
}

void GS_Problem::WriteSol(string id, string fileExt, bool postProcessSol)  const
{
	GS_Image resultImg;
		
	if(fileExt.compare("raw") == 0)
		resultImg = this->result;
	else
		resultImg = GetResultRGB(postProcessSol);

	ImageIO::WriteFile(resultImg, GetResultImgFN(id, fileExt));
}

void GS_Problem::Read(string id, string fileExt)
{
	this->problemLevels.Read(id, fileExt);
	this->input.Read(id, fileExt);
	verifyChannelsTypeCorrectness();
	ImageIO::ReadFile(this->result, GetResultImgFN(id, fileExt));
	INSIST(this->result.Shape() == this->params.problemShape.imgShape);
}

GS_Problem GS_Problem::Load(string id)
{
	Params params = Params::Load(id);
	GS_Problem prob(params);
	prob.Read(id);
	return prob;
}

GS_P_Shape GS_Problem::GetProbShape(string id)
{
	return GS_Problem::Params::Load(id).problemShape;
}

