#include "GS_Problem.h"
#include <iostream> 
#include <fstream> 
#include "OptFileParser.h"
#include "ColorHelper.h"

GS_Problem::Params GS_Problem::Params::Default()
{
	Params params;
	params.channelsType = CT_RGB;
	params.problemShape = GS_P_Shape::Default();
	params.plsParams    = GS_ProblemLevels::Params::Default();
	return params;
}

void GS_Problem::Params::Dump(string prefix)
{
	const char* prefixPtr = prefix.c_str();

	printf("\n");
	printf("%s GS Problem params:\n", prefixPtr);

	printf("%s   channels type - %s:\n", prefixPtr, ColorHelper::GetChannelsTypeStr(this->channelsType).c_str());
	
	if(this->preprocessFilters.size() > 0)
	{
		printf("\n%s   preprocessing filters:\n", prefixPtr);
		for(uint iFlt = 0; iFlt < this->preprocessFilters.size(); iFlt++)
		{
			this->preprocessFilters[iFlt].Dump(prefix + "   ");
		}
	}

	if(this->postprocessFilters.size() > 0)
	{
		printf("\n%s   postprocessing filters:\n", prefixPtr);
		for(uint iFlt = 0; iFlt < this->postprocessFilters.size(); iFlt++)
		{
			this->postprocessFilters[iFlt].Dump(prefix + "   ");
		}
	}

	this->problemShape.Dump(prefix + "  ");
	this->plsParams.Dump(prefix + "  ");

	printf("\n");
}

void GS_Problem::Params::Write(string id) const
{
	string fileName = id + "-params.txt";
	ofstream outStream(fileName.c_str());
	outStream << "GSP.channelsType = " << ColorHelper::GetChannelsTypeStr(this->channelsType) << "\n";
	for(uint iFlt = 0; iFlt < this->preprocessFilters.size(); iFlt++)
	{
		this->preprocessFilters[iFlt].Write(outStream, "GSP.preProcess.");
	}
	for(uint iFlt = 0; iFlt < this->postprocessFilters.size(); iFlt++)
	{
		this->postprocessFilters[iFlt].Write(outStream, "GSP.postProcess.");
	}
	this->problemShape.Write(outStream, "GSP.");
	this->plsParams.Write(outStream, "GSP.");
	outStream.close();
}

GS_Problem::Params GS_Problem::Params::Load(string id)
{
	string paramsFN = id + "-params.txt";
	char *extraParams = NULL;
	OptFileParser parser;
	OptFileParser::OptFileParams optFileParams = parser.Load(paramsFN.c_str(), &extraParams, false);
	return optFileParams.gspParams;
}

