#include "GS_ProblemInput.h"
#include "ImageIO.h"
#include "Disk.h"

void GS_ProblemInput::Read(string id, string fileExt)
{
	this->imgFN = GetInputImgFN(id, fileExt);
	ImageIO::ReadFile(this->img, this->imgFN);
	INSIST(this->img.Shape() == this->shape.imgShape);

	ImageIO::ReadFile(this->data,  GetDataImgFN(  id, fileExt));
	ImageIO::ReadFile(this->dataW, GetDataW_ImgFN(id, fileExt));
}

void GS_ProblemInput::Write(string id, string fileExt) const
{
	bool rawMode = false;
	if(fileExt.compare("raw") == 0)
		rawMode = true;

	if(rawMode)
	{
		ImageIO::WriteFile(this->img,   GetInputImgFN( id, fileExt));
		ImageIO::WriteFile(this->data,  GetDataImgFN(  id, fileExt));
	}
	else
	{
		ImageIO::WriteFile(ColorHelper::ConvertChannels(CT_RGB, this->img),   GetInputImgFN( id, fileExt));
		ImageIO::WriteFile(ColorHelper::ConvertChannels(CT_RGB, this->data),  GetDataImgFN(  id, fileExt));
	}
	ImageIO::WriteFile(this->dataW, GetDataW_ImgFN(id, fileExt));

	if(this->IsEdgeMetricsDefined())
	{
		CShape elVisShape = this->emData.edgeLen.Shape();
		elVisShape.nBands = 3;
		CFloatImage elVis(elVisShape);
		//bug - just write one channel
		ImageProcessing::CopyChannel(elVis, this->emData.edgeLen, 0, 0);
		ImageProcessing::CopyChannel(elVis, this->emData.edgeLen, 1, 0);
		ImageProcessing::CopyChannel(elVis, this->emData.edgeLen, 2, 0);
		ImageProcessing::Operate(elVis, elVis, 1500.0f, ImageProcessing::OP_DIV);
		ImageIO::WriteFile(elVis, GetEdgeLenFN(id, "tga"));
	}
}

string GS_ProblemInput::GetInputImgFN(string id, string fileExt) const
{
	string baseDir = Disk::GetBaseDirName(id);
	string baseID  = Disk::GetBaseFileName(id);
	return baseDir + "input-" + baseID + "." + fileExt;
}

string GS_ProblemInput::GetEdgeLenFN(string id, string fileExt) const
{
	string baseDir = Disk::GetBaseDirName(id);
	string baseID  = Disk::GetBaseFileName(id);
	return baseDir + "el-" + baseID + "." + fileExt;
}

string GS_ProblemInput::GetDataImgFN(string id, string fileExt) const
{
	string baseDir = Disk::GetBaseDirName(id);
	string baseID  = Disk::GetBaseFileName(id);
	return baseDir + "data-" + baseID + "." + fileExt;
}

string GS_ProblemInput::GetDataW_ImgFN(string id, string fileExt) const
{
	string baseDir = Disk::GetBaseDirName(id);
	string baseID  = Disk::GetBaseFileName(id);
	return baseDir + "dataW-" + baseID + "." + fileExt;
}
