#ifndef __GS_PROBLEM_INPUT_H__
#define __GS_PROBLEM_INPUT_H__

#include "GS_Problem_Defs.h"
#include "EdgeMetrics.h"
#include <iostream> 
#include <fstream> 

typedef struct _GS_P_Shape
{
	CShape imgShape;

	_GS_P_Shape(CShape _imgShape = CShape())
	{		
		this->imgShape = _imgShape;
	}

	bool operator==(const _GS_P_Shape &ref)
	{		
		return (this->imgShape == ref.imgShape);
	}

	static _GS_P_Shape Default()
	{
		_GS_P_Shape probShape;
		probShape.imgShape = CShape(0, 0, 0);
		return probShape;
	}

	void Dump(string prefix)
	{
		const char* prefixPtr = prefix.c_str();
		printf("\n");
		printf("%s Problem shape - [%i, %i, %i]\n", prefixPtr, 
			imgShape.width, imgShape.height, imgShape.nBands);
		printf("\n");
	}

	void Write(ofstream &outStream, string paramPrefix) const
	{
		outStream << paramPrefix << "probShape = " 	
				  << this->imgShape.width  << " " 
				  << this->imgShape.height << " " 
				  << this->imgShape.nBands << "\n";
	}
} GS_P_Shape;

class GS_ProblemInput
{
public:
	typedef struct _Params
	{
		bool loadOnlyIfExists;
		bool loadEmData;
		bool loadFlowField;
		static _Params Default();
		void Dump(string prefix);
	} Params;

private:
	GS_P_Shape shape;

//depends - Copy, ExtractChannel
private:
	string            imgFN;
	HashedPtrs        auxDataObjs;
public:
	GS_Image          img;	
	GS_Image          data;
	GS_Image          dataW;
	EdgeMetrics       emData;
	CFloatImage       flowField;

public:
	GS_ProblemInput(GS_P_Shape _shape = GS_P_Shape());
	GS_ProblemInput(string _imgFN, Params loadParams = Params::Default(), ChannelsType channelsType = CT_RGB);
	GS_ProblemInput(const GS_Image& _img, ChannelsType channelsType = CT_RGB);

	GS_P_Shape Shape() const;
	void ReAllocate(GS_P_Shape _shape);
	void Copy(const GS_ProblemInput &cpySrc);
	void Copy(const GS_ProblemInput &cpySrc, int srcChannel);

	GS_ProblemInput ExtractChannel(int iChannel);

	void AddAuxData(string inputName, void *auxData, bool overwriteData = false);
	void* GetAuxData(string dataName);
	bool IsAuxDataDefined(string dataName);

	bool IsEdgeMetricsDefined() const;
	bool IsFlowFieldDefined() const;

	string GetFileName()
	{
		INSIST(this->imgFN.size() > 0);
		return this->imgFN;
	}	
	
private:
	void resize(GS_P_Shape _shape);	
	void setupDefaultData();

public: //GS_ProblemInput-IO.cpp
	void Read(string id, string fileExt = "tga");
	void Write(string id, string fileExt = "tga") const;

private:
	string GetInputImgFN(string id, string fileExt) const;
	string GetDataImgFN(string id, string fileExt) const;
	string GetDataW_ImgFN(string id, string fileExt) const;
	string GetEdgeLenFN(string id, string fileExt) const;


	friend class GS_Problem;
};

#endif

