#ifndef __GS_PROBLEM_LEVELS_H__
#define __GS_PROBLEM_LEVELS_H__

#include "GS_Problem_Defs.h"
#include "GS_ProblemLevel.h"
#include "GS_ProblemInput.h"
#include <set>
#include <iostream> 
#include <fstream> 

class GS_ProblemLevels 
{
public:
	typedef enum
	{
		LT_DOG,     //difference of gaussians
		LT_DOB,     //difference of bilateral-filters
		LT_DEFAULT = LT_DOG
	} LevelsType;

	typedef struct _Params
	{
		int levelCount;
		LevelsType levelsType;

		float sigmaD;
		float sigmaD_mult;
		float sigmaS;
		float sigmaS_mult;

		static _Params Default();
		void Dump(string prefix);
		void Write(ofstream &outStream, string paramPrefix) const;
		bool operator==(const _Params _params);
	} Params;

	static const int MinLevelsInResidue        = 2;
	static const int MinLevelsInProbForResidue = MinLevelsInResidue + 1;

private:
	Params params;
	CShape imgShape;
	vector<GS_ProblemLevel> levels;
	GS_ProblemLevel residualLevel;

public:
	GS_ProblemLevels(Params _params = Params::Default(), CShape _imgShape = CShape());
	void Init(Params _params, CShape _imgShape);
	void Resize(CShape _imgShape, int levelCount);
	void ResetLevels(GS_ProblemInput &input);
	GS_ProblemLevel* GetLevel(int iLevel);
	void Copy(const GS_ProblemLevels& cpySrc, int srcChannel);
	
	int GetLevelCount() const
	{
		return (int) this->levels.size();
	}

	void ResetResidual(const Indices &levelsNotToUse);

	void Write(string prefix, string fileExt = "tga", Indices levelsToUse = Indices());
	void Read(string prefix, string fileExt = "raw");

private:
	

	friend class GS_Problem;
};

#endif

