#include "GS_VSolver.h"

void GS_VSolver::SolveOutOfCore(Params params)
{

	int frameCount = (int) params.imgFNs.size();
	INSIST(frameCount > 0);

	if((params.inCoreFrameLimit <= 0) ||
	   (params.inCoreFrameLimit >= frameCount))
	{
		params.inCoreFrameLimit = frameCount;
		params.outOfCoreIters   = 1;
	}
	INSIST(params.outOfCoreIters >= 1);

	for(int iIter = 0; iIter < params.outOfCoreIters; iIter++)
	{
		for(int iFrame = 0; iFrame < frameCount; /* iFrame incremented at the bottom*/)
		{
			int partitionSize = min((int)(frameCount - iFrame), params.inCoreFrameLimit);
			if((iFrame == 0) && (iIter % 2 == 1))
			{
				INSIST(params.inCoreFrameLimit < frameCount);
				partitionSize = params.inCoreFrameLimit / 2;
			}

			printf("Solving frames: %i - %i\n", iFrame, iFrame + partitionSize - 1);

			GS_VSolver::Params partitionParams = params;
			partitionParams.imgFNs.clear();

			for(int i = 0; i < partitionSize; i++)
			{
				partitionParams.imgFNs.push_back(params.imgFNs[i + iFrame]);
			}

			if(iFrame > 0)
			{
				partitionParams.leftConstraintImgFN = params.imgFNs[iFrame - 1];
				partitionParams.useLeftConstraint = true;
			}
			else
			{
				partitionParams.leftConstraintImgFN = "";
				partitionParams.useLeftConstraint = false;
			}

			if((iIter > 0) &&
			   (iFrame + partitionSize < frameCount - 1))
			{
				partitionParams.rightConstraintImgFN = params.imgFNs[iFrame + partitionSize];
				partitionParams.useRightConstraint = true;
			}
			else
			{	
				partitionParams.rightConstraintImgFN = "";
				partitionParams.useRightConstraint = false;
			}

			GS_VSolver gsvSolver(partitionParams);
			gsvSolver.Solve();
			printf("\n");

			iFrame += partitionSize;
		}
	}
}
