#include "GS_VSolver.h"
#include "ImageProcessing.h"
#include "ImageIO.h"
#include "Curve.h"

void GS_VSolver::VisEdges()
{
	REPORT("Visualization edges...\n");

	GS_Image edgeVisImg(this->frameShape);	

	Curve lenCurve;
	lenCurve.Load("500 0.000000e+000 1.000000e+000 6 0.000000e+000 0.000000e+000  1.344086e-001 5.524862e-003  2.903226e-001 2.209945e-002  4.516129e-001 1.546961e-001  6.129032e-001 6.077348e-001  1.000000e+000 1.000000e+000 ");

	INSIST(this->frameShape.nBands <= 3);

	INSIST(this->frameCount > 0);
	for(int iFrame = 0; iFrame < this->frameCount; iFrame++)
	{
		printf("."); fflush(stdout);

		ImageIO::ReadFile(edgeVisImg, getSolFN(this->params.imgFNs[iFrame]));
		INSIST(edgeVisImg.Shape() == this->frameShape);		

		edgeVisImg = ColorHelper::ConvertChannels(CT_LAB, edgeVisImg);

		string edgeDataFN = getEdgeFN(this->params.imgFNs[iFrame]);
		EdgeMetrics emData;
		emData.Load(edgeDataFN, this->frameShape);

		CFloatImage edgeLenImg = emData.edgeLen;
		ImageProcessing::Operate(edgeLenImg, edgeLenImg, 1500.0f, ImageProcessing::OP_DIV);
		edgeLenImg = ImageProcessing::GaussianBlur(edgeLenImg, 5, 2.0f);

		uint nodeAddr  = 0;
		uint pixelAddr = 0;
		for(int y = 0; y < this->frameShape.height; y++)
		for(int x = 0; x < this->frameShape.width;  x++, nodeAddr++, pixelAddr += this->frameShape.nBands)
		{
			float alpha = edgeLenImg[nodeAddr];
			SET_IF_LT(alpha, 1.0f);
			alpha = (float) lenCurve.GetCurveResponse(alpha);
			//alpha /= 2.0f;
			float alphaInv = 1.0f - alpha;
			INSIST(alphaInv >= 0.0f);

			edgeVisImg[pixelAddr] = (alphaInv * edgeVisImg[pixelAddr]); 
		}

		edgeVisImg = ColorHelper::ConvertChannels(CT_RGB, edgeVisImg);
		string edgeVisFN = getEdgeVisFN(this->params.imgFNs[iFrame]);
		ImageIO::WriteFile(edgeVisImg, edgeVisFN);
	}
	printf("\n");

	exit(0);
}
