#include "GS_VSolver.h"
#include "ImageIO.h"

void GS_VSolver::initGlobals()
{
	this->frameCount = (int) this->params.imgFNs.size();
	INSIST(this->frameCount > 0);

	this->varsInFrame.resize(     this->frameCount, 0);
	this->varsInPrevFrames.resize(this->frameCount, 0);
	this->levelsInFrame.resize(   this->frameCount, 0);

	int defChannelIndex = 0;
	if(this->params.channelsToSolve.size() > 0)
		defChannelIndex = *(this->params.channelsToSolve.begin());

	this->frameShape   = ImageIO::GetImageShape(this->params.imgFNs[0]);
	string frameProbID = getProbID(this->params.imgFNs[0], defChannelIndex);
	GS_Problem::Params frameProbParam = GS_Problem::Params::Load(frameProbID);
	switch(frameProbParam.channelsType)
	{
	case CT_LOG_LUM:
	case CT_LUM:
		this->frameShape.nBands = 1;
		break;
	default:
		break;
	}


	this->maskShape    = CShape(this->frameShape.width, this->frameShape.height, 1);
	this->flowShape    = CShape(this->frameShape.width, this->frameShape.height, 3);
	this->pixelsInFrameChannel = this->frameShape.NodeCount();
	INSIST(this->pixelsInFrameChannel > 0);

	this->currToPrevFlow.ReAllocate(this->flowShape);
	this->nextToCurrFlow.ReAllocate(this->flowShape);

	if(this->params.channelsToSolve.size() == 0)
	{		
		for(int iChannel = 0; iChannel < this->frameShape.nBands; iChannel++)
		{
			this->params.channelsToSolve.insert(iChannel);
		}
	}	
	else
	{
		for(Indices::const_iterator iChannel = this->params.channelsToSolve.begin();
			iChannel != this->params.channelsToSolve.end();
			iChannel++)
		{
			CHECK_RANGE(*iChannel, 0, this->frameShape.nBands - 1);
		}
	}
}

void GS_VSolver::initChannel(int iChannel)
{
	for(int iFrame = 0; iFrame < frameCount; iFrame++)
	{
		string frameProbID = getProbID(this->params.imgFNs[iFrame], iChannel);
		GS_Problem::Params frameProbParam = GS_Problem::Params::Load(frameProbID);
		INSIST(frameProbParam.problemShape.imgShape == this->maskShape);

		this->levelsInFrame[iFrame] = frameProbParam.plsParams.levelCount;
		this->varsInFrame[iFrame]   = this->levelsInFrame[iFrame] * this->pixelsInFrameChannel;
		if(iFrame > 0)
			this->varsInPrevFrames[iFrame] = this->varsInPrevFrames[iFrame - 1] + this->varsInFrame[iFrame - 1];
		else
			this->varsInPrevFrames[iFrame] = 0;		
	}
}

int GS_VSolver::getVarsInProb()
{
	INSIST(this->frameCount > 0);
	return this->varsInPrevFrames[this->frameCount - 1] + this->varsInFrame[this->frameCount - 1];
}

