#ifndef __GS_VSOLVER_H__
#define __GS_VSOLVER_H__

#include "GS_Solver_Defs.h"
#include "SparseSolver.h"
#include "GS_Solver.h"
#include <map>

class GS_VSolver
{
public:
	typedef struct _Params : public GS_Solver::Params
	{
		double fidelity_tGrad;
		bool useFlow, useFlowCnf;
		double flowCnfPow;

		bool   useLeftConstraint;
		string leftConstraintImgFN;

		bool   useRightConstraint;
		string rightConstraintImgFN;

		int inCoreFrameLimit;
		int outOfCoreIters;

		vector<string> imgFNs;		

		void Dump(string prefix);
		static _Params Default();
	} Params;

private:
	typedef std::map<uint,  float> VarCoefList;
	typedef std::pair<uint, float> VarCoef;
	typedef std::pair<VarCoefList::iterator, bool> VarInsertResult;

private:
	Params      params;
	int         frameCount;
	int         pixelsInFrameChannel;
	CShape      frameShape;
	CShape      maskShape;
	CShape      flowShape;

	vector<int> levelsInFrame;
	vector<int> varsInFrame;
	vector<int> varsInPrevFrames;

	GS_Problem  prevFrameProb;
	GS_Problem  currFrameProb;
	GS_Problem  nextFrameProb;

	CFloatImage currToPrevFlow;
	CFloatImage nextToCurrFlow;

	uint getVarAddr(uint nodeAddr, int iProblemLevel, int iFrame)
	{
		return nodeAddr + (iProblemLevel * this->pixelsInFrameChannel) + this->varsInPrevFrames[iFrame];
	}

public:
	GS_VSolver(const Params &_params);
	void Solve();	
	
private:
	void solve(int iChannel);
	void solveChannel(int iChannel);
	void saveChannelSol(int iChannel, SparseSolver& solver);
	void loadEqs(int iChannel, SparseSolver& solver);
	void defineEqs(int iChannel, int iProbLevel, int iFrame, 
				   VarCoefList* neighsInPrevFrame, VarCoefList* neighsInNextFrame,
				   SparseSolver& solver);

private: //GS_VSolver-Init.cpp
	void initGlobals();
	void initChannel(int iChannel);
	int getVarsInProb();

public: //GS_VSolver-IO.cpp
	void CombineResults();
private: 
	void loadFrameContext(int iFrame, int iChannel);
	void loadFrame(int iFrame, int iChannel);
	string getFlowFN(string imgFN); 
	string getEdgeFN(string imgFN); 
	string getProbID(string imgFN, int iChannel); 
	string getSolFN(string imgFN);
	string getEdgeVisFN(string imgFN);

private: //GS_VSolver-CollectNeighs.cpp
	void collectSpatialNeighbours(VarCoefList* neighsInFrame);

	void collectTemporalNeighbours(VarCoefList* neighsInFrame, 
								    const CFloatImage& outTemporalFlow, bool collectOutNeighbours,
								    const CFloatImage& inTemporalFlow,  bool collectInNeighbours);

private: //GS_VSolver-TGrads.cpp
	void setTGrad_default(double& tGrad, double& tGradWeight); 
	void setTGrad_temporalCoherence(double& tGrad, double& tGradWeight); 
	void setTGrad_deflicker(double fxy, double& tGrad, double& tGradWeight); 
	void setTGrad_sharpenMotion(double& tGrad, double& tGradWeight); 

public: //GS_VSolver-Driver.cpp
	static void SolveOutOfCore(Params params);


public: //GS_VSolver-Edges.cpp
	void VisEdges();
};


#endif

