#include "LinearSolver.h"
#include "mkl_rci.h"
#include "mkl_blas.h"
#include "mkl_spblas.h"
#include "SparseSymMat_CSR.h"

#ifdef __USE_MKL__ 

LinearSolRet 
LinearSolver::Solve_PCG_mkl(size_v n, AbstractMat* A, precise* x, precise* b, 
				  	        Params params, AbsractPreconditioner* MInv)
{

	MKL_INT rci_request;
	MKL_INT ipar[128];
	double dpar[128];

	double* tmp = new double[n * 4];
	INSIST(tmp != NULL);
	precise* searchDir  = tmp + (0 * n);
	precise* matProdSol = tmp + (1 * n);	
	precise* residual   = tmp + (2 * n);
	precise* precondSol = tmp + (3 * n);


	dcg_init(&n, x, b, &rci_request, ipar, dpar, tmp);
	INSIST(rci_request == 0);

	//http://www.intel.com/software/products/mkl/docs/webhelp/ssr/ssr_RCIInterfaceDesc.html#rci_id_CommonParameters
	ipar[4] = params.maxIters;
	ipar[8] = 1;
	ipar[9] = 0;
	dpar[0] = params.errTolerance;
	if(MInv != NULL)
	{
		ipar[10] = 1;
	}

	char precondMatdes[3];
	precondMatdes[0]='d';
	precondMatdes[1]='l';
	precondMatdes[2]='n';
	precise precondAlpha = (precise) 1.0;

	//needed for the jacobi precond
	//SparseSymMat_CSR* symMat = (SparseSymMat_CSR*) A;

	dcg_check(&n, x, b, &rci_request, ipar, dpar, tmp);
	INSIST(rci_request == 0);

	while(true)
	{
		dcg(&n, x, b, &rci_request, ipar, dpar, tmp);
		switch(rci_request)
		{
		case -1:
			REPORT("Warning: optimization might not have coverged\n");
			goto dcg_done;
			break;

		case 0:
			goto dcg_done;
			break;

		case 1:
			A->Multiply(n , n, searchDir, matProdSol);
			break;

		case 3:
			INSIST(MInv != NULL);
			memcpy(precondSol, residual, sizeof(*tmp) * n);
			
			//mkl_dcsrsv(&precondMatdes[2], &n, &precondAlpha, precondMatdes, 
			//		   symMat->values, symMat->columns, symMat->rowIndices, symMat->rowIndices + 1, 
			//		   residual, precondSol);
		
			MInv->Apply(n, precondSol);
			break;

		default:
			REPORT_FAILURE("Unknown rci_request - %i\n", rci_request);
			break;
		}
	}

dcg_done:
	int iterCount;
	dcg_get(&n, x, b, &rci_request, ipar, dpar, tmp, &iterCount);

	delete [] tmp;

	return LinearSolRet(precise(dpar[4]), iterCount);
}

#endif

