#pragma once

#include "Definitions.h"
#include "LinearSolverDefs.h"
#include "AbstractMat.h"
#include "AbsractPreconditioner.h"
#include "Vec.h"

typedef std::pair<precise,int> LinearSolRet; //<last err, iterations>

class LinearSolver
{

public:
	typedef struct _Params
	{
		precise errTolerance;
		int maxIters;
		int fltErrMaxiters;

		static _Params Default();
	}Params;

	//Solve linear system using conjugate gradients
	static LinearSolRet Solve_CG(size_v n, AbstractMat* A, precise* x, precise* b, 
				     		     Params params = Params::Default());

	//Solve linear system using preconditioned conjugate gradients
	static LinearSolRet Solve_PCG(size_v n, AbstractMat* A, precise* x, precise* b, 
							 	  Params params = Params::Default(),
								  AbsractPreconditioner* MInv = NULL);

	static precise GetResidual(int n, AbstractMat* A, precise* x, precise* b);

public: //LinearSolver-Pro.cpp
	static LinearSolRet Solve_CG_pro(size_v n, AbstractMat* A, precise* x, precise* b, 
				     				 Params params = Params::Default());

	//Solve linear system using preconditioned conjugate gradients
	static LinearSolRet Solve_PCG_pro(size_v n, AbstractMat* A, precise* x, precise* b, 
							 		  Params params = Params::Default(),
									  AbsractPreconditioner* MInv = NULL);

public: //LinearSolver-MKL.cpp
	//Solve linear system using preconditioned conjugate gradients
	static LinearSolRet Solve_PCG_mkl(size_v n, AbstractMat* A, precise* x, precise* b, 
							 		  Params params = Params::Default(),
									  AbsractPreconditioner* MInv = NULL);

public: //LinearSolver-Debug.cpp
	static void SolveRandSys_GD(int size);
};



