#include "SparseSqrSymMat.h"


SparseSqrSymMat::SparseSqrSymMat(size_v _n)
{
	this->n = _n;
	INSIST(this->matData.size() == 0);
	MatRow emptyRow;
	this->matData.resize(this->n, emptyRow);
}

SparseSqrSymMat::~SparseSqrSymMat()
{
}

size_vv 
SparseSqrSymMat::GetNonZeroCount() const
{
	size_vv nonZeroCount = 0;

	for(vector<MatRow>::const_iterator currRow = this->matData.begin();
		currRow != this->matData.end();
		currRow++)
	{
		nonZeroCount += (size_vv) currRow->size();
	}

	return nonZeroCount;
}

void 
SparseSqrSymMat::Multiply(size_v inLen, size_v outLen, 
							   const precise* inVec, precise* outVec) const
{
	INSIST(inLen  == this->n);
	INSIST(outLen == this->n);

	Vec::SetToNull(outLen, outVec);

	size_v iRow = 0;
	for(vector<MatRow>::const_iterator currRow = this->matData.begin();
		currRow != this->matData.end();
		currRow++, iRow++)
	{			
		for(MatRow::const_iterator currCol = currRow->begin(); 
			currCol != currRow->end(); 
			currCol++)
		{
			size_v iCol = currCol->first;
			outVec[iRow] += inVec[iCol] * currCol->second;
			if(iCol != iRow)
			{
				outVec[iCol] += inVec[iRow] * currCol->second;
			}
		}
	}
}


