#ifndef __SPARSE_MAT_H__
#define __SPARSE_MAT_H__

#include "Definitions.h"
#include "LinearSolverDefs.h"
#include "AbstractMat.h"
#include "Vec.h"
#include <map>
#include "SparseSymMat_CSR.h"

class SparseSqrSymMat : public AbstractMat
{
	typedef std::map<size_v, precise> MatRow;
	vector<MatRow> matData; 
	size_v n;

public:
	SparseSqrSymMat(size_v _n = 0);
	~SparseSqrSymMat();

	void SetRow(size_v iRow, const precise* rowValues, const 
			    size_v* rowColumns, size_vv rowNonZeroCount)
	{
		for(size_v iElem = 0; iElem < rowNonZeroCount; iElem++)
		{
			SetElem(iRow, rowColumns[iElem], rowValues[iElem]);
		}
	}

	void SetRow(size_v iRow, const vector<precise>& rowValues, 
		        const vector<size_v>& rowColumns)
	{
		size_vv rowNonZeroCount = (size_vv) rowValues.size();
		INSIST(rowColumns.size() == (size_t) rowNonZeroCount);

		for(size_v iElem = 0; iElem < rowNonZeroCount; iElem++)
		{
			SetElem(iRow, rowColumns[iElem], rowValues[iElem]);			
		}
	}

	void SetElem(size_v iRow, size_v iCol, precise val)
	{
		if(val == 0) return;

		INSIST(iRow <= iCol);
		CHECK_RANGE(iRow, 0, this->n - 1);
		CHECK_RANGE(iCol, 0, this->n - 1);

		this->matData[iRow][iCol] = val;
	}

	precise GetElem(size_v iRow, size_v iCol) const
	{
		CHECK_RANGE(iRow, 0, this->n - 1);
		CHECK_RANGE(iCol, 0, this->n - 1);
		if(iRow > iCol)
		{
			size_v temp = iRow;
			iRow = iCol;
			iCol = temp;
		}

		MatRow::const_iterator findResult = this->matData[iRow].find(iCol);
		if(findResult == this->matData[iRow].end())
			return 0;
		else
			return findResult->second;
	}

	size_v GetRowCount() const
	{
		return this->n;
	}

	size_v GetColCount() const
	{
		return this->n;
	}

	size_vv GetNonZeroCount() const;

	void Multiply(size_v inLen, size_v outLen, 
				  const precise* inVec, precise* outVec) const;

public: //SparseSqrSymMat-Debug.cpp
	void Dump();
	void DumpMatlab(const char* matName = "A");
	static void DumpRandVecMult(size_v size);
	static SparseSqrSymMat GenRandMat(size_v size);
	static SparseSqrSymMat GenIdenMat(size_v size);

	SparseSymMat_CSR* ConvertToCSR();
};

#endif


