#include "SparseSolver_pro.h"
#include "LinearSolver.h"
#include "Timer.h"

SparseSolver_pro::SparseSolver_pro(size_v _varCount, SparseSolverParams _params)
{
	this->varCount = _varCount;
	this->params   = _params;

	if(this->params.nonZeroInMatGuess == 0)
	{
		this->params.nonZeroInMatGuess = this->varCount * 7;
	}
	INSIST(this->params.nonZeroInMatGuess > 0);
	this->aMat = new SparseSymMat_CSR(varCount, varCount, this->params.nonZeroInMatGuess);
	INSIST(this->aMat != NULL);

	this->xVec = new precise[this->varCount];
	INSIST(this->xVec != NULL);

	this->bVec = new precise[this->varCount];
	INSIST(this->bVec != NULL);

	this->precond = NULL;
	if(this->params.usePrecondHB)
	{
		this->precond = new PrecondHBF_stack(this->varCount, 
										     this->params.imgShapes, 
											 this->params.maxLevelsInPrecond);
	}
}

SparseSolver_pro::~SparseSolver_pro() 
{
	delete [] this->xVec;
	delete [] this->bVec;
	delete this->aMat;

	if(this->precond != NULL)
		delete this->precond;
}

void 
SparseSolver_pro::Solve(double accuracy, int maxIter, bool verbose) 
{
	Timer timer;
	if(verbose)
	{
		REPORT("Solving System...\n");
	}

	LinearSolver::Params params = LinearSolver::Params::Default();
	params.errTolerance = (float) accuracy;
	params.maxIters = maxIter;
	std::pair<precise, int> returnVal;
	
	if(this->precond == NULL)
	{
		returnVal = LinearSolver::Solve_CG(this->varCount, 
										   this->aMat, this->xVec, this->bVec, 
										   params);	
	}
	else
	{
		returnVal = LinearSolver::Solve_PCG(this->varCount, 
										    this->aMat, this->xVec, this->bVec, 
										    params, this->precond);
	}

	if(verbose)
	{
		REPORT("\tDone - %.3e in %.2lf(s) and %i(iters).\n", (double) returnVal.first, 
															 timer.GetSecondsCount(),
															 returnVal.second);
	}
}
