#include "GSF_Colorize.h"
#include "ImageIO.h"
#include "Disk.h"

GS_Problem::Params GSF_Colorize::GS_Problem_Params_Default()
{
	GS_Problem::Params params = GS_Problem::Params::Default();
	params.plsParams.levelCount = 1;
	params.channelsType = CT_YUV;
	return params;
}

GS_Solver::Params GSF_Colorize::GS_Solver_Params_Default()
{
	GS_Solver::Params params = GS_Solver::Params::Default();
	
	params.channelsToSolve.insert(1);
	params.channelsToSolve.insert(2);
	params.maxIters = 500;
	params.pinchFuncExp = GSF_Colorize::Params::Default().edgeSensitivity;
	params.fidelity_data = 1;
	return params;
}


GS_Image GSF_Colorize::Colorize(GS_ProblemInput probInput,
								Params fltParams,
								GS_Problem::Params probParams,
								GS_Solver::Params solParams)
{
	string paramsSpacing((TAB_SPACE_COUNT + 1) * TAB_LEVEL, ' ');
	
	REPORT("Creating GS problem... \n");
	probParams.Dump(paramsSpacing);
	GS_Problem prob(probInput, probParams);	

	REPORT("Filtering GS problem... \n");
	fltParams.Dump(paramsSpacing);
	GSF_Colorize fltFunc(&prob, fltParams);
	if(solParams.levelsToSolve.size() == 0)
		fltFunc.Filter();
	else
		fltFunc.Filter(solParams.levelsToSolve);

	REPORT("Solving GS problem... \n");
	solParams.Dump(paramsSpacing);
	GS_Solver solver(&prob, solParams);
	solver.Solve();

	return prob.GetResultRGB();
}


void GSF_Colorize::TestFilters(string imgFN, string outDir)
{
	GSF_Colorize::Params fltParams          = Params::Default();
	GS_Problem::Params probParams           = GS_Problem_Params_Default();
	GS_Solver::Params solverParams          = GS_Solver_Params_Default();
	GS_ProblemInput::Params probInputParams = GS_ProblemInput::Params::Default();	
	if(fltParams.improvedSDI == true)
	{
		probInputParams.loadEmData = true;
		solverParams.useSDI_weighing = true;
	}
	GS_ProblemInput probInput(imgFN, probInputParams, probParams.channelsType);	

	INSIST(solverParams.pinchFuncExp == fltParams.edgeSensitivity);	

	string fileID = Disk::GetFileNameID(Disk::GetBaseFileName(imgFN));
	GS_Image fltImg;

	fltImg = GSF_Colorize::Colorize(probInput, fltParams, probParams, solverParams);
	ImageIO::WriteFile(fltImg, outDir + "colorize-default-" + fileID + ".tga");
}
