#include "GSF_Colorize.h"
#include "ImageProcessing.h"
#include "Disk.h"
#include "ImageIO.h"

GSF_Colorize::GSF_Colorize(GS_Problem* _problem, const Params &_params) : GS_Filter(_problem)
{
	this->params = _params;

	GS_ProblemInput* probInput = this->problem->GetInputBuffer();
	INSIST(probInput->img.channelsType == CT_YUV);

	if(this->params.improvedSDI)
	{
		INSIST(probInput->IsEdgeMetricsDefined() == true);
	}

	string orgFN      = probInput->GetFileName();
	string scriblesFN = getScribblesFN(orgFN);

	CShape imgShape = probInput->Shape().imgShape;
	INSIST(imgShape.nBands == 3);

	GS_Image scribblesImg;
	ImageIO::ReadFile(scribblesImg, scriblesFN);
	INSIST(imgShape == scribblesImg.Shape());
	scribblesImg = ColorHelper::ConvertChannels(CT_YUV, scribblesImg);

	ImageProcessing::Operate(probInput->data,  probInput->data,  0.0f, ImageProcessing::OP_SET, 1);
	ImageProcessing::Operate(probInput->data,  probInput->data,  0.0f, ImageProcessing::OP_SET, 2);
	ImageProcessing::Operate(probInput->dataW, probInput->dataW, 0.0f, ImageProcessing::OP_SET, 1);
	ImageProcessing::Operate(probInput->dataW, probInput->dataW, 0.0f, ImageProcessing::OP_SET, 2);
	ImageProcessing::CopyChannel(probInput->img,  probInput->img,  1, 0);
	ImageProcessing::CopyChannel(probInput->img,  probInput->img,  2, 0);

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{
		if((fabs(scribblesImg[pixelAddr + 1]) > 0.0005f) ||
		   (fabs(scribblesImg[pixelAddr + 2]) > 0.0005f))
		{
			probInput->data[pixelAddr + 1] = scribblesImg[pixelAddr + 1];
			probInput->data[pixelAddr + 2] = scribblesImg[pixelAddr + 2];

			probInput->dataW[pixelAddr + 1] = 1.0E+0;
			probInput->dataW[pixelAddr + 2] = 1.0E+0;
		}
	}
}

void GSF_Colorize::FilterLevel(int iLevel, void* userData)
{	
	Params* levelParams = (userData == NULL) ? &this->params : (Params*) userData;
	
	GS_ProblemLevel* levelProb = this->problem->GetLevel(iLevel);

	GS_Image levelOrg,  levelFlt;
	GS_Image levelData, levelDataW;
	GS_Image levelOrgGF_X, levelOrgGF_Y;
	GS_Image levelFltGF_X, levelFltGF_Y;
	levelProb->GetData(levelOrg,  levelFlt,
		               levelData, levelDataW,
					   levelOrgGF_X, levelOrgGF_Y, levelFltGF_X, levelFltGF_Y);

	GS_ProblemInput* probInput = this->problem->GetInputBuffer();

	if((levelParams->bfSpaceS > FLT_EPSILON) && 
	   (levelParams->bfSigS   > FLT_EPSILON))
	{
		CShape imgShape = levelOrg.Shape();
		GS_Image lumImg = ImageProcessing::ExtractChannel(levelOrg, 0);
		lumImg = ImageProcessing::BilateralFilter(lumImg, levelParams->bfSpaceS, levelParams->bfSigS);

		int pixelAddr      = 0;
		int nodeAddr       = 0;
		int leftNodeAddr   = -1;
		int bottomNodeAddr = -imgShape.width;
		for(int y = 0; y < imgShape.height; y++)
		for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands, nodeAddr++, leftNodeAddr++, bottomNodeAddr++)
		{
			imgType currPixel = lumImg[nodeAddr];
			if(x > 0)
				levelOrgGF_X[pixelAddr] = currPixel - lumImg[leftNodeAddr];
			else
				levelOrgGF_X[pixelAddr] = 0;
			if(y > 0)
				levelOrgGF_Y[pixelAddr] = currPixel - lumImg[bottomNodeAddr];
			else
				levelOrgGF_Y[pixelAddr] = 0;
		}
	}

	for(int iChannel = 1; iChannel <= 2; iChannel++)
	{
		ImageProcessing::Operate(levelFltGF_X, levelFltGF_X, 0.0f, ImageProcessing::OP_SET, iChannel);
		ImageProcessing::Operate(levelFltGF_Y, levelFltGF_Y, 0.0f, ImageProcessing::OP_SET, iChannel);
		ImageProcessing::CopyChannel(levelOrgGF_X, levelOrgGF_X, iChannel, 0);
		ImageProcessing::CopyChannel(levelOrgGF_Y, levelOrgGF_Y, iChannel, 0);
		ImageProcessing::CopyChannel(levelData,  probInput->data,  iChannel, iChannel);
		ImageProcessing::CopyChannel(levelDataW, probInput->dataW, iChannel, iChannel);

		//check
		ImageProcessing::CopyChannel(levelOrg,   probInput->img,   iChannel, 0);

		//check
		//ImageProcessing::CopyChannel(levelFlt,   probInput->data,  iChannel, iChannel);
	}
}

string GSF_Colorize::getScribblesFN(string orgFN)
{
	return Disk::GetFileNameID(orgFN) + "-scribbles." + Disk::GetFileNameExt(orgFN);
}

