#include "GSF_Deblock.h"
#include "ImageProcessing.h"
#include "Disk.h"
#include "ImageIO.h"

GSF_Deblock::GSF_Deblock(GS_Problem* _problem, const Params& _params) : GS_Filter(_problem)
{
	this->params = _params;

	GS_ProblemInput* probInput = this->problem->GetInputBuffer();
	//bug
	//INSIST(probInput->img.channelsType == CT_YUV);
	if(probInput->img.channelsType == CT_YUV)
	{
		INSIST(this->params.uvMult >= 0.0f);
		ImageProcessing::Operate(probInput->dataW, probInput->dataW, (imgType) this->params.uvMult, ImageProcessing::OP_MUL, 1);
		ImageProcessing::Operate(probInput->dataW, probInput->dataW, (imgType) this->params.uvMult, ImageProcessing::OP_MUL, 2);
	}
}

void GSF_Deblock::FilterLevel(int iLevel, void* userData)
{	
	
	Params* levelParams = (userData == NULL) ? &this->params : (Params*) userData;

	GS_ProblemLevel* levelProb = this->problem->GetLevel(iLevel);

	GS_Image levelOrg,     levelFlt;
	GS_Image levelData,    levelDataW;
	GS_Image levelOrgGF_X, levelOrgGF_Y;
	GS_Image levelFltGF_X, levelFltGF_Y;
	levelProb->GetData(levelOrg,  levelFlt, 
					   levelData, levelDataW,
					   levelOrgGF_X, levelOrgGF_Y, levelFltGF_X, levelFltGF_Y);

	CShape imgShape = levelOrgGF_X.Shape();
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++)
	for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++, pixelAddr++)
	{
		if((x % 8) == 0)
		{
			imgType blockGradMag = (imgType) fabs(levelOrgGF_X[pixelAddr]);
			if(levelOrgGF_X[pixelAddr] < 0)
				levelFltGF_X[pixelAddr] = (imgType) (-levelParams->blockGradCurve.GetCurveResponse(blockGradMag));
			else
				levelFltGF_X[pixelAddr] = (imgType) (levelParams->blockGradCurve.GetCurveResponse(blockGradMag));
		}

		int yy = imgShape.height - 1 - y;
		if((yy % 8) == 7)
		{
			imgType blockGradMag = (imgType) fabs(levelOrgGF_Y[pixelAddr]);
			if(levelOrgGF_Y[pixelAddr] < 0)
				levelFltGF_Y[pixelAddr] = (imgType) (-levelParams->blockGradCurve.GetCurveResponse(blockGradMag));
			else
				levelFltGF_Y[pixelAddr] = (imgType) (levelParams->blockGradCurve.GetCurveResponse(blockGradMag));
		}
	}

	ImageProcessing::Operate(levelDataW, levelDataW, (imgType) levelParams->uvMult, ImageProcessing::OP_MUL, 1);
	ImageProcessing::Operate(levelDataW, levelDataW, (imgType) levelParams->uvMult, ImageProcessing::OP_MUL, 2);
}


