#ifndef __GS_ELE_H__
#define __GS_ELE_H__

#include "GS_Filter_Defs.h"
#include "GS_Filter.h"

//ELE = Enhance long edges
class GSF_ELE : public GS_Filter
{
public:
	typedef struct _Params 
	{
		float multiplier;
		float edgeLenNorm;

		_Params(float _multiplier = 0.8f, float _edgeLenNorm = 1500.0f)
		{
			this->multiplier  = _multiplier;
			this->edgeLenNorm = _edgeLenNorm;
		}
		static _Params Default();
		static _Params DefaultInv();
		void Dump(string prefix);
	}Params;

private:
	Params params;

public:
	GSF_ELE(GS_Problem* _problem, const Params &_params = Params::Default());
	void FilterLevel(int iLevel, void* userData = NULL);

	Params GetParams()
	{
		return this->params;
	}

public: //GSF_ELE-Driver.cpp
	static GS_Problem::Params GS_Problem_Params_Default();
	static GS_Problem::Params GS_Problem_Params_MultiFreq();
	static GS_Solver::Params  GS_Solver_Params_Default();

	static GS_Image ELE(GS_ProblemInput& probInput,
						Params fltParams              = Params::Default(),
						GS_Problem::Params probParams = GS_Problem_Params_Default(),
						GS_Solver::Params solParams   = GS_Solver_Params_Default());

	static GS_Image ELE_Freq(GS_ProblemInput& probInput, int iFreqStart, int iFreqEnd,
							 Params fltParams,
							 GS_Problem::Params probParams,
							 GS_Solver::Params solParams);

	static GS_Image ELE_HiFreq(GS_ProblemInput& probInput,
							   Params fltParams              = Params::Default(),
							   GS_Problem::Params probParams = GS_Problem_Params_MultiFreq(),
							   GS_Solver::Params solParams   = GS_Solver_Params_Default());

	static GS_Image ELE_MidFreq(GS_ProblemInput& probInput,
							    Params fltParams              = Params::Default(),
							    GS_Problem::Params probParams = GS_Problem_Params_MultiFreq(),
							    GS_Solver::Params solParams   = GS_Solver_Params_Default());

	static GS_Image ELE_LowFreq(GS_ProblemInput& probInput,
							    Params fltParams              = Params::Default(),
							    GS_Problem::Params probParams = GS_Problem_Params_MultiFreq(),
							    GS_Solver::Params solParams   = GS_Solver_Params_Default());


	static void TestFilters(string imgFN, string outDir = "");
};

#endif

