#include "Filter_Driver.h"

Filter_Driver::Params Filter_Driver::Params::Default()
{
	Params params;

	params.type                 = FT_DEFAULT;
	params.gspParams            = GS_Problem::Params::Default();
	params.gssParams            = GS_Solver::Params::Default();
	params.gsf_EleParams        = GSF_ELE::Params::Default();
	params.gsf_SharpenParams    = GSF_Sharpen::Params::Default();
	params.gsf_motionBlurParams = GSF_MotionBlur::Params::Default();
	params.gsf_gradCurveParams  = GSF_GradCurve::Params::Default();
	params.gsf_nprParams        = GSF_NPR::Params::Default();	
	params.gsf_tmParams         = GSF_ToneMap::Params::Default();	
	params.gsf_clzParams        = GSF_Colorize::Params::Default();	
	params.gsf_relightParams    = GSF_Relight::Params::Default();
	params.gsf_testParams       = GSF_Test::Params::Default();	

	return params;
}

void Filter_Driver::Params::Dump(string prefix)
{
	const char* prefixPtr = prefix.c_str();

	printf("\n");
	printf("%s GS Filter params:\n", prefixPtr);

	this->gspParams.Dump(prefix + "  ");
	this->gssParams.Dump(prefix + "  ");

	printf("%s  filter type - ", prefixPtr);
	switch(this->type)
	{
	case FT_NULL:
		printf("FT_NULL\n");
		break;

	case FT_SHARPEN:
		printf("FT_SHARPEN\n");
		this->gsf_SharpenParams.Dump(prefix + "  ");
		break;

	case FT_ELE:
		printf("FT_ELE\n");
		this->gsf_EleParams.Dump(prefix + "  ");
		break;

	case FT_TEST:
		printf("FT_TEST\n");
		this->gsf_testParams.Dump(prefix + "  ");
		break;

	case FT_RELIGHT:
		printf("FT_RELIGHT\n");
		this->gsf_relightParams.Dump(prefix + "  ");
		break;

	case FT_DEBLOCK:
		printf("FT_DEBLOCK\n");
		this->gsf_deblockParams.Dump(prefix + "  ");
		break;

	case FT_MOTION_BLUR:
		printf("FT_MOTION_BLUR\n");
		this->gsf_motionBlurParams.Dump(prefix + "  ");
		break;

	case FT_GRAD_CURVE:
		printf("FT_GRAD_CURVE\n");
		this->gsf_gradCurveParams.Dump(prefix + "  ");
		break;

	case FT_NPR:
		printf("FT_NPR\n");
		this->gsf_nprParams.Dump(prefix + "  ");
		break;

	case FT_TM:
		printf("FT_TM\n");
		this->gsf_tmParams.Dump(prefix + "  ");
		break;

	case FT_COLORIZE:
		printf("FT_COLORIZE\n");
		this->gsf_clzParams.Dump(prefix + "  ");
		break;

	default:
		REPORT_FAILURE("Unknown filter type - %i\n", this->type);
		break;
	};

	printf("\n");
}



string Filter_Driver::Params::GetFilterName(FT_Type type)
{
	string fltName;

	switch(type)
	{
	case FT_NULL:
		fltName = "FT_NULL";
		break;

	case FT_SHARPEN:
		fltName = "FT_SHARPEN";
		break;

	case FT_ELE:
		fltName = "FT_ELE";
		break;

	case FT_TEST:
		fltName = "FT_TEST";
		break;

	case FT_RELIGHT:
		fltName = "FT_RELIGHT";
		break;

	case FT_DEBLOCK:
		fltName = "FT_DEBLOCK";
		break;

	case FT_MOTION_BLUR:
		fltName = "FT_MOTION_BLUR";
		break;

	case FT_GRAD_CURVE:
		fltName = "FT_GRAD_CURVE";
		break;

	case FT_NPR:
		fltName = "FT_NPR";
		break;

	case FT_TM:
		fltName = "FT_TM";
		break;

	case FT_COLORIZE:
		fltName = "FT_COLORIZE";
		break;

	default:
		REPORT_FAILURE("Unknown filter type - %i\n", type);
		break;
	};

	return fltName;
}

