#include "GSF_GradCurve.h"
#include "Disk.h"
#include "ImageIO.h"

GS_Problem::Params GSF_GradCurve::GS_Problem_Params_Default()
{
	GS_Problem::Params params = GS_Problem::Params::Default();
	params.plsParams.levelCount = 1;
	params.channelsType = CT_RGB;
	return params;
}

GS_Solver::Params GSF_GradCurve::GS_Solver_Params_Default()
{
	GS_Solver::Params params = GS_Solver::Params::Default();
	params.fidelity_data = 3.0E-02;
	return params;
}

GS_Solver::Params GSF_GradCurve::GS_Solver_Params_NPR()
{
	GS_Solver::Params params = GS_Solver::Params::Default();
	params.fidelity_data = 3.0E-05;
	return params;
}

GS_Image GSF_GradCurve::GradCurve(GS_ProblemInput probInput,
								  Params fltParams,
								  GS_Problem::Params probParams,
								  GS_Solver::Params solParams)
{
	string paramsSpacing((TAB_SPACE_COUNT + 1) * TAB_LEVEL, ' ');
	
	REPORT("Creating GS problem... \n");
	probParams.Dump(paramsSpacing);
	GS_Problem prob(probInput, probParams);	

	REPORT("Filtering GS problem... \n");
	fltParams.Dump(paramsSpacing);
	GSF_GradCurve gradCurveFlt(&prob, fltParams);
	if(solParams.levelsToSolve.size() == 0)
		gradCurveFlt.Filter();
	else
		gradCurveFlt.Filter(solParams.levelsToSolve);

	REPORT("Solving GS problem... \n");
	solParams.Dump(paramsSpacing);
	GS_Solver solver(&prob, solParams);
	solver.Solve();

	return prob.GetResultRGB();
}

void GSF_GradCurve::TestFilters(string imgFN, string outDir)
{
	GS_Problem::Params probParams           = GS_Problem::Params::Default();
	GS_ProblemInput::Params probInputParams = GS_ProblemInput::Params::Default();
	GS_ProblemInput probInput(imgFN, probInputParams, probParams.channelsType);

	string fileID = Disk::GetFileNameID(Disk::GetBaseFileName(imgFN));
	GS_Image fltImg;

	fltImg = GSF_GradCurve::GradCurve(probInput, Params::Default(), probParams);
	ImageIO::WriteFile(fltImg, outDir + "gc-default-" + fileID + ".tga");

	fltImg = GSF_GradCurve::GradCurve(probInput, Params::Sharpen(), probParams);
	ImageIO::WriteFile(fltImg, outDir + "gc-sharpen-" + fileID + ".tga");

	fltImg = GSF_GradCurve::GradCurve(probInput, Params::Blur(), probParams);
	ImageIO::WriteFile(fltImg, outDir + "gc-blur-" + fileID + ".tga");

	fltImg = GSF_GradCurve::GradCurve(probInput, Params::SmartSharpen(), probParams);
	ImageIO::WriteFile(fltImg, outDir + "gc-smartSharpen-" + fileID + ".tga");

	fltImg = GSF_GradCurve::GradCurve(probInput, Params::NPR(), probParams, GS_Solver_Params_NPR());
	ImageIO::WriteFile(fltImg, outDir + "gc-npr-" + fileID + ".tga");
}
