#include "GSF_GradCurve.h"

GSF_GradCurve::GSF_GradCurve(GS_Problem* _problem, const Params &_params) : GS_Filter(_problem)
{
	this->params = _params;
}

void GSF_GradCurve::FilterLevel(int iLevel, void* userData)
{	
	Params* levelParams = (userData == NULL) ? &this->params : (Params*) userData;

	GS_ProblemLevel* levelProb = this->problem->GetLevel(iLevel);

	GS_Image levelOrgGF_X, levelOrgGF_Y;
	GS_Image levelFltGF_X, levelFltGF_Y;
	levelProb->GetDataGF(levelOrgGF_X, levelOrgGF_Y, levelFltGF_X, levelFltGF_Y);

	Curve* gradCurve = &levelParams->gradCurve;

	CShape imgShape = levelOrgGF_X.Shape();
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++)
	for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++, pixelAddr++)
	{
		double gradMag, gradSign;
		
		if(levelOrgGF_X[pixelAddr] < 0.0f)
		{
			gradMag  = -levelOrgGF_X[pixelAddr];
			gradSign = -1.0;
		}
		else
		{
			gradMag  = levelOrgGF_X[pixelAddr];
			gradSign = 1.0;
		}
		levelFltGF_X[pixelAddr] = (imgType) (gradSign * gradCurve->GetCurveResponse(gradMag));


		if(levelOrgGF_Y[pixelAddr] < 0.0f)
		{
			gradMag  = -levelOrgGF_Y[pixelAddr];
			gradSign = -1.0;
		}
		else
		{
			gradMag  = levelOrgGF_Y[pixelAddr];
			gradSign = 1.0;
		}
		levelFltGF_Y[pixelAddr] = (imgType) (gradSign * gradCurve->GetCurveResponse(gradMag));
	}
}



