#include "GSF_MotionBlur.h"
#include "Disk.h"
#include "ImageIO.h"

GS_Problem::Params GSF_MotionBlur::GS_Problem_Params_Default()
{
	GS_Problem::Params params = GS_Problem::Params::Default();
	params.plsParams.levelCount = 1;
	params.channelsType = CT_RGB;
	return params;
}

GS_Solver::Params GSF_MotionBlur::GS_Solver_Params_Default()
{
	GS_Solver::Params params = GS_Solver::Params::Default();
	params.fidelity_data = 1.0E-002;
	return params;
}


GS_Image GSF_MotionBlur::MotionBlur(GS_ProblemInput& probInput,
								    Params fltParams,
									GS_Problem::Params probParams,
									GS_Solver::Params solParams)
{

	string paramsSpacing((TAB_SPACE_COUNT + 1) * TAB_LEVEL, ' ');
	
	REPORT("Filtering GS problem... \n");
	fltParams.Dump(paramsSpacing);
	GS_Problem prob(probInput, probParams);	

	REPORT("Filtering GS problem... \n");
	fltParams.Dump(paramsSpacing);
	GSF_MotionBlur motionBlurFlt(&prob, fltParams);
	if(solParams.levelsToSolve.size() == 0)
		motionBlurFlt.Filter();
	else
		motionBlurFlt.Filter(solParams.levelsToSolve);

	REPORT("Solving GS problem... \n");
	solParams.Dump(paramsSpacing);
	GS_Solver solver(&prob, solParams);
	solver.Solve();

	return prob.GetResultRGB();	
}


void GSF_MotionBlur::TestFilters(string imgFN, string outDir)
{
	GS_Problem::Params probParams           = GS_Problem::Params::Default();
	GS_ProblemInput::Params probInputParams = GS_ProblemInput::Params::Default();
	probInputParams.loadFlowField = true;
	GS_ProblemInput probInput(imgFN, probInputParams, probParams.channelsType);

	string fileID = Disk::GetFileNameID(Disk::GetBaseFileName(imgFN));
	GS_Image fltImg;

	Params defParams = GSF_MotionBlur::Params::Default();
	
	fltImg = GSF_MotionBlur::MotionBlur(probInput, defParams, probParams);
	ImageIO::WriteFile(fltImg, outDir + "mb-default-" + fileID + ".tga");
}
