#include "GSF_NPR.h"
#include "Disk.h"
#include "ImageIO.h"

GS_Problem::Params GSF_NPR::GS_Problem_Params_Default()
{
	GS_Problem::Params params   = GS_Problem::Params::Default();
	params.plsParams.levelCount = 1;
	params.channelsType = CT_RGB;
	params.preprocessFilters.push_back(GS_Problem::Filter::Default(PFT_SATURATE_COLOR));
	return params;
}

GS_Solver::Params GSF_NPR::GS_Solver_Params_Default()
{
	GS_Solver::Params params = GS_Solver::Params::Default();
	params.fidelity_data = 9.0e-003;
	params.pinchFuncExp  = 9.0f;
	return params;
}

GS_Image GSF_NPR::NPR(GS_ProblemInput probInput,
					  Params fltParams,
					  GS_Problem::Params probParams,
					  GS_Solver::Params solParams)
{
	string paramsSpacing((TAB_SPACE_COUNT + 1) * TAB_LEVEL, ' ');
	
	REPORT("Creating GS problem... \n");
	probParams.Dump(paramsSpacing);
	GS_Problem prob(probInput, probParams);	

	REPORT("Filtering GS problem... \n");
	fltParams.Dump(paramsSpacing);
	GSF_NPR fltFunc(&prob, fltParams);
	if(solParams.levelsToSolve.size() == 0)
		fltFunc.Filter();
	else
		fltFunc.Filter(solParams.levelsToSolve);

	REPORT("Solving GS problem... \n");
	solParams.Dump(paramsSpacing);
	GS_Solver solver(&prob, solParams);
	solver.Solve();

	return prob.GetResultRGB();
}

void GSF_NPR::TestFilters(string imgFN, string outDir)
{
	GS_Problem::Params probParams           = GS_Problem_Params_Default();
	GS_ProblemInput::Params probInputParams = GS_ProblemInput::Params::Default();
	probInputParams.loadEmData = true;
	GS_ProblemInput probInput(imgFN, probInputParams, probParams.channelsType);

	string fileID = Disk::GetFileNameID(Disk::GetBaseFileName(imgFN));
	GS_Image fltImg;

	fltImg = GSF_NPR::NPR(probInput, Params::Default(), probParams);
	ImageIO::WriteFile(fltImg, outDir + "npr-default-" + fileID + ".tga");
	
	GS_Image strokesImg1 = GSF_NPR::VisBrushStrokes1(fltImg, probInput.emData);
	GS_Image strokesImg2 = GSF_NPR::VisBrushStrokes2(fltImg, probInput.emData);
	ImageIO::WriteFile(strokesImg1, outDir + "npr-strokes1-" + fileID + ".tga");
	ImageIO::WriteFile(strokesImg2, outDir + "npr-strokes2-" + fileID + ".tga");
}
