#include "GSF_NPR.h"
#include "ColorHelper.h"
#include "ImageProcessing.h"

GS_Image GSF_NPR::VisBrushStrokes(const GS_Image &baseImg, const EdgeMetrics &emData, 
								  Params params)
{
	return GSF_NPR::VisBrushStrokes2(baseImg, emData, FVector3(0.0f, 0.0f, 0.0f), params);
}


GS_Image GSF_NPR::VisBrushStrokes1(const GS_Image &baseImg, const EdgeMetrics &emData, 
							      Params params)
{
	CShape imgShape = baseImg.Shape();
	INSIST(imgShape.SameIgnoringNBands(emData.edgeLen.Shape()));	
	INSIST(emData.edgeLen.Shape().nBands == 1);
	INSIST(emData.edgeLen.Shape() == emData.edgeStrength.Shape());

	GS_Image resultImg = ColorHelper::ConvertChannels(CT_YUV, baseImg);

	CFloatImage edgeLenImg = emData.edgeLen.Clone();
	ImageProcessing::Operate(edgeLenImg, edgeLenImg, params.edgeLenNorm, ImageProcessing::OP_DIV);
	edgeLenImg = ImageProcessing::LocalMaxImg(edgeLenImg, 3);
	edgeLenImg = ImageProcessing::GaussianBlur(edgeLenImg, 5, 2.0f);

	uint nodeAddr  = 0;
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, nodeAddr++, pixelAddr += imgShape.nBands)
	{
		float alpha = edgeLenImg[nodeAddr];
		SET_IF_LT(alpha, 1.0f);
		alpha = (float) params.edgeLenCurve.GetCurveResponse(alpha);		
		SET_IF_LT(alpha, 1.0f);
		alpha *= emData.edgeStrength[nodeAddr] / 255.0f;
		alpha *= 1.5f;

		resultImg[pixelAddr] += (alpha * resultImg[pixelAddr]); 
	}

	resultImg = ColorHelper::ConvertChannels(CT_RGB, resultImg);
	return resultImg;
}


GS_Image GSF_NPR::VisBrushStrokes2(const GS_Image &baseImg, const EdgeMetrics &emData, 
							       const FVector3 &strokeCol, Params params)
{
	CShape imgShape = baseImg.Shape();
	INSIST(imgShape.SameIgnoringNBands(emData.edgeLen.Shape()));
	INSIST(emData.edgeLen.Shape().nBands == 1);
	INSIST(emData.edgeLen.Shape() == emData.edgeStrength.Shape());
	INSIST(imgShape.nBands <= 3);

	GS_Image resultImg(imgShape);

	CFloatImage edgeLenImg = emData.edgeLen.Clone();
	ImageProcessing::Operate(edgeLenImg, edgeLenImg, params.edgeLenNorm, ImageProcessing::OP_DIV);	
	edgeLenImg = ImageProcessing::LocalMaxImg(edgeLenImg, 3);
	edgeLenImg = ImageProcessing::GaussianBlur(edgeLenImg, 5, 2.0f);

	uint nodeAddr  = 0;
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, nodeAddr++, pixelAddr += imgShape.nBands)
	{
		float alpha = edgeLenImg[nodeAddr];
		SET_IF_LT(alpha, 1.0f);
		alpha = (float) params.edgeLenCurve.GetCurveResponse(alpha);
		SET_IF_LT(alpha, 1.0f);
		alpha *= emData.edgeStrength[nodeAddr] / 255.0f;
		alpha *= 1.5f;

		float alphaInv = 1.0f - alpha;
		for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++)
		{
			resultImg[pixelAddr + iChannel] = (alphaInv * baseImg[pixelAddr + iChannel]) + 
				                              (alpha    * strokeCol[iChannel]); 
		}
	}

	return resultImg;
}
