#include "GSF_NPR.h"
#include "EdgeMetrics.h"
#include "GSF_GradCurve.h"
#include "ImageProcessing.h"
#include "ImageIO.h"

GSF_NPR::GSF_NPR(GS_Problem* _problem, const Params &_params) : GS_Filter(_problem)
{this->params = _params;
}

void GSF_NPR::FilterLevel(int iLevel, void* userData)
{	
	INSIST(this->problem->GetInputBuffer()->IsEdgeMetricsDefined() == true);	

	Params* levelParams = (userData == NULL) ? &this->params : (Params*) userData;

	GS_ProblemLevel* levelProb = this->problem->GetLevel(iLevel);

	CFloatImage emLen = this->problem->GetInputBuffer()->emData.edgeLen.Clone();
	ImageProcessing::Operate(emLen, emLen, levelParams->edgeLenNorm, ImageProcessing::OP_DIV);
	emLen = ImageProcessing::LocalMaxImg(emLen, 5);	
	emLen = ImageProcessing::LocalMinImg(emLen, 5);	
	emLen = ImageProcessing::GaussianBlur(emLen, 5, 1.0f);

	CFloatImage emAng = this->problem->GetInputBuffer()->emData.edgeDirc;

	Curve& edgeLenCurve = levelParams->edgeLenCurve;

	GS_Image levelOrgGF_X, levelOrgGF_Y;
	GS_Image levelFltGF_X, levelFltGF_Y;
	levelProb->GetDataGF(levelOrgGF_X, levelOrgGF_Y, levelFltGF_X, levelFltGF_Y);

	CShape imgShape = levelOrgGF_X.Shape();
	uint pixelAddr = 0;
	uint nodeAddr  = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, nodeAddr++)
	{
		float edgeLength  = emLen[nodeAddr];
		SET_IF_LT(edgeLength, 1.0f);
		float gradMult = (float) edgeLenCurve.GetCurveResponse(edgeLength);

		float edgeAngRad  = emAng[nodeAddr];
		float gradWeightX = pow(cos(edgeAngRad), 2.0f);
		float gradWeightY = 1 - gradWeightX; 

		for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++, pixelAddr++)
		{
			levelFltGF_X[pixelAddr] = gradMult * levelOrgGF_X[pixelAddr] * gradWeightX;
			levelFltGF_Y[pixelAddr] = gradMult * levelOrgGF_Y[pixelAddr] * gradWeightY;
		}
	}
}


