#include "GSF_Relight.h"
#include "ImageProcessing.h"
#include "Disk.h"
#include "ImageIO.h"


GSF_Relight::GSF_Relight(GS_Problem* _problem, const Params& _params) : GS_Filter(_problem)
{
	this->params = _params;
}


void GSF_Relight::FilterLevel(int iLevel, void* userData)
{	
	float angVar = 1.0f;

	Params* levelParams = (userData == NULL) ? &this->params : (Params*) userData;

	GS_ProblemLevel* levelProb = this->problem->GetLevel(iLevel);

	GS_Image levelOrgGF_X, levelOrgGF_Y;
	GS_Image levelFltGF_X, levelFltGF_Y;
	levelProb->GetDataGF(levelOrgGF_X, levelOrgGF_Y, levelFltGF_X, levelFltGF_Y);

	CShape imgShape = levelOrgGF_X.Shape();
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++)
	for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++, pixelAddr++)
	{
		float ux      = levelOrgGF_X[pixelAddr];
		float uy      = levelOrgGF_Y[pixelAddr];
		float gradAng = MathHelper::GetAngInDegrees(ux, uy);

		float localLightDircAng;

		switch(levelParams->lightType)
		{
		case LT_DIRC:
			localLightDircAng = this->params.lightDircAng;
			break;

		case LT_POINT:
			if(this->params.lightPointInv)
				localLightDircAng = MathHelper::GetAngInDegrees(x - levelParams->lightPoint[0], 
																y - levelParams->lightPoint[1]);
			else
				localLightDircAng = MathHelper::GetAngInDegrees(levelParams->lightPoint[0] - x, 
																levelParams->lightPoint[1] - y);
			break;

		default:
			printf("Unknown light type - %i\n", levelParams->lightType);
			break;
		}

		float angDiffRad = MathHelper::DegressToRadians(gradAng - localLightDircAng);
		float gradWeight = exp(angDiffRad * angDiffRad / -angVar) * levelParams->lightStrength;
		
		levelFltGF_X[pixelAddr] = ux + (ux * gradWeight);
		levelFltGF_Y[pixelAddr] = uy + (uy * gradWeight);
	}
}
