#include "PrecondHBF.h"

#define ImageIndex(x,y) (((y) * this->width) + (x))

PrecondHBF::PrecondHBF(int _width, int _height, int _levels)
{
	this->width  = _width;
	this->height = _height;
	this->levels = _levels;

	this->nodeCount = this->width * this->height;
	INSIST(this->nodeCount > 0);

	if(this->levels == -1)
	{
		this->levels = PrecondHBF::guessNumberOfLevels(this->width, this->height);
	}
}


int PrecondHBF::guessNumberOfLevels(int _width, int _height)
{
	INSIST(_width  > 0);
	INSIST(_height > 0);

	int minPow2 = -1; // 2^minPow2 <= _width,_height
	while((_width > 0) && (_height > 0))
	{
		_width  >>= 1;
		_height >>= 1;
		minPow2++;		
	}

	int levels = minPow2 - 1;
	INSIST(levels > 1);
	return levels;
}

void PrecondHBF::sweepUp(precise* vec) const
{
			
}

void PrecondHBF::sweepDown(precise* vec) const
{
	for(int iLevel = this->levels - 2;  iLevel >= 0; iLevel--)
	{
		sweepDown(vec, iLevel);
	}	
}

void PrecondHBF::sweepUp(precise* vec, int level) const
{
			
}

void PrecondHBF::sweepDown(precise* vec, int targetLevel) const
{
	int neighStep = 1 << targetLevel;
	int nodeStep  = neighStep << 1;

	for(int y = neighStep; y < this->height - neighStep; y += nodeStep)
	for(int x = 0;         x < this->width;              x += nodeStep)	
	{
		vec[nodeIndex(x,y)] += 0.5f * vec[nodeIndex(x, y - neighStep)] +
			                   0.5f * vec[nodeIndex(x, y + neighStep)];
	}

	for(int y = 0;         y < this->height;             y += nodeStep)
	for(int x = neighStep; x < this->width  - neighStep; x += nodeStep)	
	{
		vec[nodeIndex(x,y)] += 0.5f * vec[nodeIndex(x - neighStep, y)] +
			                   0.5f * vec[nodeIndex(x + neighStep, y)];
	}

	for(int y = neighStep; y < this->height - neighStep; y += nodeStep)
	for(int x = neighStep; x < this->width  - neighStep; x += nodeStep)	
	{
		vec[nodeIndex(x,y)] += 0.25f * vec[nodeIndex(x - neighStep, y)] +
			                   0.25f * vec[nodeIndex(x + neighStep, y)] + 
							   0.25f * vec[nodeIndex(x, y - neighStep)] +
			                   0.25f * vec[nodeIndex(x, y + neighStep)];
	}
}
