#include "ColorHelper.h"
#include <stdlib.h>
#include <stdio.h>
#include <float.h>
#include "ColorHelper.h"
#include "ImageProcessing.h"

CTransform3x3 ColorHelper::rgbTolmsMAT;
CTransform3x3 ColorHelper::lmsTolabMAT;
CTransform3x3 ColorHelper::lmsTorgbMAT;
CTransform3x3 ColorHelper::labTolmsMAT;

ColorHelper::_InitColorHelper ColorHelper::_InitColorHelper::init;

CFloatImage ColorHelper::RGBtoLuminance(CFloatImage &src)
{
	INSIST(src.channelsType == CT_RGB);

	CShape imgShape = src.Shape();
	if(imgShape.nBands == 1)
	{
		return src;
	}
	else
	{
		ENSURE(imgShape.nBands == 3);
	}
	CFloatImage lumImage(imgShape.width, imgShape.height, 1);

	uint colorPixelAddr = 0;
	uint lumPixelAddr   = 0;
	for(int y = 0; y < imgShape.height; y++)
	{
		for(int x = 0; x < imgShape.width; x++, colorPixelAddr += imgShape.nBands, lumPixelAddr++)
		{
			float lum = 0.0f;
			lum += src[colorPixelAddr + 0] * 0.299f;
			lum += src[colorPixelAddr + 1] * 0.587f;
			lum += src[colorPixelAddr + 2] * 0.114f;
			lumImage[lumPixelAddr] = lum;
		}
	}

	return lumImage;
}


CFloatImage ColorHelper::NormalizeRGBbyLum(const CFloatImage& rgbImg, const CFloatImage& lumImg, float sat)
{
	CShape rgbShape = rgbImg.Shape();
	INSIST(rgbShape.SameIgnoringNBands(lumImg.Shape()));
	INSIST(rgbShape.nBands == 3);
	INSIST(lumImg.Shape().nBands == 1);
	INSIST(rgbImg.channelsType == CT_RGB);
	INSIST(lumImg.channelsType == CT_LUM);

	CFloatImage rgbImgScaled = rgbImg.Clone();
	CFloatImage lumImgScaled = lumImg.Clone();
	ImageProcessing::Rescale(rgbImgScaled, 0.0f, 1.0f);
	ImageProcessing::Rescale(lumImgScaled, 0.0f, 1.0f);
	
	CFloatImage normalizedImg(rgbShape);

	uint colorPixelAddr = 0;
	uint lumPixelAddr   = 0;
	for(int y = 0; y < rgbShape.height; y++)
	for(int x = 0; x < rgbShape.width;  x++, colorPixelAddr += rgbShape.nBands, lumPixelAddr++)
	{
		float lumIn = 0.0f;
		lumIn += rgbImgScaled[colorPixelAddr + 0] * 0.299f;
		lumIn += rgbImgScaled[colorPixelAddr + 1] * 0.587f;
		lumIn += rgbImgScaled[colorPixelAddr + 2] * 0.114f;
		
		if(lumIn == 0.0f)
			lumIn = LogLumE;

		normalizedImg[colorPixelAddr + 0] = pow(rgbImgScaled[colorPixelAddr + 0] / lumIn, sat) * lumImgScaled[lumPixelAddr];
		normalizedImg[colorPixelAddr + 1] = pow(rgbImgScaled[colorPixelAddr + 1] / lumIn, sat) * lumImgScaled[lumPixelAddr];
		normalizedImg[colorPixelAddr + 2] = pow(rgbImgScaled[colorPixelAddr + 2] / lumIn, sat) * lumImgScaled[lumPixelAddr];
	}

	return normalizedImg;
}

void ColorHelper::RGBtoLum(CFloatImage &lumImg, const CFloatImage &rgbImg)
{
	CShape imgShape = rgbImg.Shape();
	ENSURE(imgShape.SameIgnoringNBands(lumImg.Shape()));
	ENSURE(imgShape.nBands == 3);
	ENSURE(lumImg.Shape().nBands == 1);

	INSIST(rgbImg.channelsType == CT_RGB);
	lumImg.channelsType = CT_LUM;

	uint pixelAddr = 0;
	uint lumAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands, lumAddr++)
	{ 
		float lum = 0.0f;
		lum += rgbImg[pixelAddr + 0] * 0.299f;
		lum += rgbImg[pixelAddr + 1] * 0.587f;
		lum += rgbImg[pixelAddr + 2] * 0.114f;
		lumImg[lumAddr] = lum;
	}
}

void ColorHelper::LumtoRGB(CFloatImage &rgbImg, const CFloatImage &lumImg)
{
	CShape imgShape = rgbImg.Shape();
	ENSURE(imgShape.SameIgnoringNBands(lumImg.Shape()));
	ENSURE(imgShape.nBands == 3);
	ENSURE(lumImg.Shape().nBands == 1);

	rgbImg.channelsType = CT_RGB;
	INSIST(lumImg.channelsType == CT_LUM);

	ImageProcessing::CopyChannel(rgbImg, lumImg, 0, 0);
	ImageProcessing::CopyChannel(rgbImg, lumImg, 1, 0);
	ImageProcessing::CopyChannel(rgbImg, lumImg, 2, 0);	
}

void ColorHelper::LumtoLogLum(CFloatImage &logLumImg, const CFloatImage &lumImg)
{
	CShape imgShape = lumImg.Shape();
	ENSURE(imgShape == logLumImg.Shape());
	ENSURE(imgShape.nBands == 1);

	INSIST(lumImg.channelsType == CT_LUM);
	logLumImg.channelsType = CT_LOG_LUM;

	uint pixelCount = imgShape.PixelCount();
	for(uint iPixel = 0; iPixel < pixelCount; iPixel++)
	{
		logLumImg[iPixel] = log(lumImg[iPixel] + LogLumE);
	}
}

void ColorHelper::LogLumtoLum(CFloatImage &lumImg, const CFloatImage &logLumImg)
{
	CShape imgShape = lumImg.Shape();
	ENSURE(imgShape == logLumImg.Shape());
	ENSURE(imgShape.nBands == 1);

	lumImg.channelsType = CT_LUM;
	INSIST(logLumImg.channelsType == CT_LOG_LUM);

	uint pixelCount = imgShape.PixelCount();
	for(uint iPixel = 0; iPixel < pixelCount; iPixel++)
	{
		 lumImg[iPixel] = exp(logLumImg[iPixel]) - LogLumE;
		 //SET_IF_GT(lumImg[iPixel], 0.0f);
		 //SET_IF_LT(lumImg[iPixel], 1.0f);
	}
}


void ColorHelper::RgbtoLogRgb(CFloatImage &logRgbImg, const CFloatImage &rgbImg)
{
	CShape imgShape = rgbImg.Shape();
	ENSURE(imgShape == logRgbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(rgbImg.channelsType == CT_RGB);
	logRgbImg.channelsType = CT_LOG_RGB;

	uint pixelCount = imgShape.PixelCount();
	for(uint iPixel = 0; iPixel < pixelCount; iPixel++)
	{
		logRgbImg[iPixel] = log(rgbImg[iPixel] + LogLumE);
	}
}

void ColorHelper::LogRgbtoRgb(CFloatImage &rgbImg, const CFloatImage &logRgbImg)
{
	CShape imgShape = rgbImg.Shape();
	ENSURE(imgShape == logRgbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	rgbImg.channelsType = CT_RGB;
	INSIST(logRgbImg.channelsType == CT_LOG_RGB);

	uint pixelCount = imgShape.PixelCount();
	for(uint iPixel = 0; iPixel < pixelCount; iPixel++)
	{
		 rgbImg[iPixel] = exp(logRgbImg[iPixel]) - LogLumE;
		 //SET_IF_GT(lumImg[iPixel], 0.0f);
		 //SET_IF_LT(lumImg[iPixel], 1.0f);
	}
}

void ColorHelper::RGBtoLogYCrCb(CFloatImage &logYCrCbImg, const CFloatImage &rgbImg)
{
	CShape imgShape = rgbImg.Shape();
	ENSURE(imgShape == logYCrCbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(rgbImg.channelsType == CT_RGB);

	ColorHelper::RGBtoYCbCr(logYCrCbImg, rgbImg);
	logYCrCbImg.channelsType = CT_LOG_YCrCb;
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{ 
		logYCrCbImg[pixelAddr + 0] = log(logYCrCbImg[pixelAddr + 0] + LogLumE);
	}
}

void ColorHelper::LogYCrCbtoRGB(CFloatImage& rgbImg, const CFloatImage& logYCrCbImg)
{
	CShape imgShape = rgbImg.Shape();
	ENSURE(imgShape == logYCrCbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	rgbImg.channelsType = CT_RGB;
	INSIST(logYCrCbImg.channelsType == CT_LOG_YCrCb);

	CFloatImage yCrCbImg(imgShape);
	yCrCbImg.channelsType = CT_YCrCb;
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{ 
		yCrCbImg[pixelAddr + 0] = exp(logYCrCbImg[pixelAddr + 0]) - LogLumE;
	}

	ColorHelper::YCbCrtoRGB(rgbImg, yCrCbImg);
}



void ColorHelper::RGBtoLogYUV(CFloatImage &logYuvImg, const CFloatImage &rgbImg)
{
	CShape imgShape = rgbImg.Shape();
	ENSURE(imgShape == logYuvImg.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(rgbImg.channelsType == CT_RGB);

	ColorHelper::RGBtoYUV(logYuvImg, rgbImg);
	logYuvImg.channelsType = CT_LOG_YUV;
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{ 
		logYuvImg[pixelAddr + 0] = log(logYuvImg[pixelAddr + 0] + LogLumE);
	}
}

void ColorHelper::LogYUVtoRGB(CFloatImage &rgbImg, const CFloatImage &logYuvImg)
{
	CShape imgShape = rgbImg.Shape();
	ENSURE(imgShape == logYuvImg.Shape());
	ENSURE(imgShape.nBands == 3);

	rgbImg.channelsType = CT_RGB;
	INSIST(logYuvImg.channelsType == CT_LOG_YUV);

	CFloatImage yuvImg(imgShape);
	yuvImg.channelsType = CT_YUV;
	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{ 
		yuvImg[pixelAddr + 0] = exp(logYuvImg[pixelAddr + 0]) - LogLumE;
	}

	ColorHelper::YUVtoRGB(rgbImg, yuvImg);
}


void ColorHelper::RGBtoLogLum(CFloatImage &logLumImg, const CFloatImage &rgbImg)
{
	CFloatImage lumImg(logLumImg.Shape());
	ColorHelper::RGBtoLum(lumImg, rgbImg);
	ColorHelper::LumtoLogLum(logLumImg, lumImg);
}

void ColorHelper::LogLumtoRGB(CFloatImage &rgbImg, const CFloatImage &logLumImg)
{
	CFloatImage lumImg(logLumImg.Shape());
	ColorHelper::LogLumtoLum(lumImg, logLumImg);
	ColorHelper::LumtoRGB(rgbImg, lumImg);
}

void ColorHelper::RGBtoHSV(CFloatImage &dstHSV_Img, const CFloatImage &srcRGB_Img)
{
	CShape imgShape = srcRGB_Img.Shape();
	ENSURE(imgShape == dstHSV_Img.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(srcRGB_Img.channelsType == CT_RGB);
	dstHSV_Img.channelsType = CT_HSV;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{ 
		ColorHelper::RGBtoHSV(srcRGB_Img[pixelAddr + 0],  srcRGB_Img[pixelAddr + 1],  srcRGB_Img[pixelAddr + 2],
							  &dstHSV_Img[pixelAddr + 0], &dstHSV_Img[pixelAddr + 1], &dstHSV_Img[pixelAddr + 2]);
	}
}

void ColorHelper::HSVtoRGB(CFloatImage &dstRGB_Img, const CFloatImage &srcHSV_Img)
{
	CShape imgShape = srcHSV_Img.Shape();
	ENSURE(imgShape == dstRGB_Img.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(srcHSV_Img.channelsType == CT_HSV);
	dstRGB_Img.channelsType = CT_RGB;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{ 
		ColorHelper::HSVtoRGB(&dstRGB_Img[pixelAddr + 0], &dstRGB_Img[pixelAddr + 1], &dstRGB_Img[pixelAddr + 2],
							   srcHSV_Img[pixelAddr + 0],  srcHSV_Img[pixelAddr + 1],  srcHSV_Img[pixelAddr + 2]);
	}
}

void ColorHelper::RGBtoLAB(CFloatImage &dst, const CFloatImage &src)
{
	CShape imgShape = dst.Shape();
	ENSURE(imgShape == src.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(src.channelsType == CT_RGB);
	dst.channelsType = CT_LAB;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	{
		for(int x = 0; x < imgShape.width; x++, pixelAddr += imgShape.nBands)
		{
			CVector3 rgbVec;
			rgbVec[0] = src[pixelAddr + 0];
			rgbVec[1] = src[pixelAddr + 1];
			rgbVec[2] = src[pixelAddr + 2];

			CVector3 lmsVec = ColorHelper::rgbTolmsMAT * rgbVec;
			
			lmsVec[0] = log(lmsVec[0] + LogLumE);
			lmsVec[1] = log(lmsVec[1] + LogLumE);
			lmsVec[2] = log(lmsVec[2] + LogLumE);

			CVector3 labVec = ColorHelper::lmsTolabMAT * lmsVec;
			dst[pixelAddr + 0] = (float) labVec[0];
			dst[pixelAddr + 1] = (float) labVec[1];
			dst[pixelAddr + 2] = (float) labVec[2];
		}
	}
}

void ColorHelper::LABtoRGB(CFloatImage &dst, const CFloatImage &src)
{
	CShape imgShape = dst.Shape();
	ENSURE(imgShape == src.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(src.channelsType == CT_LAB);
	dst.channelsType = CT_RGB;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	{
		for(int x = 0; x < imgShape.width; x++, pixelAddr += imgShape.nBands)
		{
			CVector3 labVec;
			labVec[0] = src[pixelAddr + 0];
			labVec[1] = src[pixelAddr + 1];
			labVec[2] = src[pixelAddr + 2];

			CVector3 lmsVec = ColorHelper::labTolmsMAT * labVec;

			lmsVec[0] = exp(lmsVec[0]) - LogLumE;
			lmsVec[1] = exp(lmsVec[1]) - LogLumE;
			lmsVec[2] = exp(lmsVec[2]) - LogLumE;

			CVector3 rgbVec = ColorHelper::lmsTorgbMAT * lmsVec;
			dst[pixelAddr + 0] = max( min((float) rgbVec[0], 1.0f), 0.0f );
			dst[pixelAddr + 1] = max( min((float) rgbVec[1], 1.0f), 0.0f );
			dst[pixelAddr + 2] = max( min((float) rgbVec[2], 1.0f), 0.0f );
		}
	}
}


void ColorHelper::RGBtoHSV( float r, float g, float b, float *h, float *s, float *v )
{
	float min, max, delta;

	min = std::min(std::min(r, g), b);
	max = std::max(std::max(r, g), b);
	*v = max;				// v

	delta = max - min;

	if( max != 0 )
		*s = delta / max;		// s
	else {
		// r = g = b = 0		// s = 0, v is undefined
		*s = 0;
		*h = -1;
		return;
	}

	if( r == max )
		*h = ( g - b ) / delta;		// between yellow & magenta
	else if( g == max )
		*h = 2 + ( b - r ) / delta;	// between cyan & yellow
	else
		*h = 4 + ( r - g ) / delta;	// between magenta & cyan

	*h *= 60;				// degrees
	if( *h < 0 )
		*h += 360;

}

void ColorHelper::HSVtoRGB( float *r, float *g, float *b, float h, float s, float v )
{
	int i;
	float f, p, q, t;

	if( s == 0 ) {
		// achromatic (grey)
		*r = *g = *b = v;
		return;
	}

	h /= 60;			// sector 0 to 5
	i = (int) floor( h );
	f = h - i;			// factorial part of h
	p = v * ( 1 - s );
	q = v * ( 1 - s * f );
	t = v * ( 1 - s * ( 1 - f ) );

	switch( i ) {
		case 0:
			*r = v;
			*g = t;
			*b = p;
			break;
		case 1:
			*r = q;
			*g = v;
			*b = p;
			break;
		case 2:
			*r = p;
			*g = v;
			*b = t;
			break;
		case 3:
			*r = p;
			*g = q;
			*b = v;
			break;
		case 4:
			*r = t;
			*g = p;
			*b = v;
			break;
		default:		// case 5:
			*r = v;
			*g = p;
			*b = q;
			break;
	}

}

// conversation formula - http://www.jpeg.org/public/jfif.pdf
void ColorHelper::RGBtoYCbCr(CFloatImage &yCbCrImg, const CFloatImage &rgbImg)
{
	CShape imgShape = yCbCrImg.Shape();
	ENSURE(imgShape == rgbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(rgbImg.channelsType == CT_RGB);
	yCbCrImg.channelsType = CT_YCrCb;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{
		float r = rgbImg[pixelAddr + 0];
		float g = rgbImg[pixelAddr + 1];
		float b = rgbImg[pixelAddr + 2];
		yCbCrImg[pixelAddr + 0] = ( 0.2990f * r) + (0.5870f * g) + (0.1140f * b);
		yCbCrImg[pixelAddr + 1] = (-0.1687f * r) - (0.3313f * g) + (0.5000f * b) + 0.5f;
		yCbCrImg[pixelAddr + 2] = ( 0.5000f * r) - (0.4187f * g) - (0.0813f * b) + 0.5f;
	}
}

// conversation formula - http://www.jpeg.org/public/jfif.pdf
void ColorHelper::YCbCrtoRGB(CFloatImage &rgbImg,  const CFloatImage &yCbCrImg)
{
	CShape imgShape = yCbCrImg.Shape();
	ENSURE(imgShape == rgbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(yCbCrImg.channelsType == CT_YCrCb);
	rgbImg.channelsType = CT_RGB;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{
		float y =  yCbCrImg[pixelAddr + 0];
		float cb = yCbCrImg[pixelAddr + 1];
		float cr = yCbCrImg[pixelAddr + 2];
		rgbImg[pixelAddr + 0] = y + (1.40200f * (cr - 0.5f));
		rgbImg[pixelAddr + 1] = y - (0.34414f * (cb - 0.5f)) - (0.71414f * (cr - 0.5f));
		rgbImg[pixelAddr + 2] = y + (1.77200f * (cb - 0.5f));
	}
}

void ColorHelper::RGBtoYUV(CFloatImage &yuvImg, const CFloatImage &rgbImg)
{
	CShape imgShape = yuvImg.Shape();
	ENSURE(imgShape == rgbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(rgbImg.channelsType == CT_RGB);
	yuvImg.channelsType = CT_YUV;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{
		float r = rgbImg[pixelAddr + 0];
		float g = rgbImg[pixelAddr + 1];
		float b = rgbImg[pixelAddr + 2];
		yuvImg[pixelAddr + 0] = ( 0.2990f * r) + (0.5870f * g) + (0.1140f * b);
		yuvImg[pixelAddr + 1] = (-0.14713f * r) - (0.28886f * g) + (0.436f * b);
		yuvImg[pixelAddr + 2] = ( 0.615f * r) - (0.51499f * g) - (0.10001f * b);
	}

}

void ColorHelper::YUVtoRGB(CFloatImage &rgbImg, const CFloatImage &yuvImg)
{
	CShape imgShape = yuvImg.Shape();
	ENSURE(imgShape == rgbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(yuvImg.channelsType == CT_YUV);
	rgbImg.channelsType = CT_RGB;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{
		float y = yuvImg[pixelAddr + 0];
		float u = yuvImg[pixelAddr + 1];
		float v = yuvImg[pixelAddr + 2];

		rgbImg[pixelAddr + 0] = y + (1.13983f * v);
		rgbImg[pixelAddr + 1] = y - (0.39465f * u) - (0.58060f * v);
		rgbImg[pixelAddr + 2] = y + (2.03211f * u);
	}
}

//ffm = ffMpeg
//coverter works for yuv images generated by ffmpeg
void ColorHelper::ffmYUVtoRGB(CFloatImage &rgbImg, const CFloatImage &ffm_yuvImg)
{
	CShape imgShape = ffm_yuvImg.Shape();
	ENSURE(imgShape == rgbImg.Shape());
	ENSURE(imgShape.nBands == 3);

	INSIST(ffm_yuvImg.channelsType == CT_YUV);
	rgbImg.channelsType = CT_RGB;

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, pixelAddr += imgShape.nBands)
	{
		float y = ffm_yuvImg[pixelAddr + 0] * 255;
		float u = ffm_yuvImg[pixelAddr + 1] * 255;
		float v = ffm_yuvImg[pixelAddr + 2] * 255;

		float C = y - FFM_I_OFFSET;
		float D = u - FFM_UV_OFFSET;
		float E = v - FFM_UV_OFFSET;

		rgbImg[pixelAddr + 0] = ((298 * C           + 409 * E + 128) / 256);
		rgbImg[pixelAddr + 1] = ((298 * C - 100 * D - 208 * E + 128) / 256);
		rgbImg[pixelAddr + 2] = ((298 * C + 516 * D           + 128) / 256);

		rgbImg[pixelAddr + 0] /= 255;
		rgbImg[pixelAddr + 1] /= 255;
		rgbImg[pixelAddr + 2] /= 255;
	}
}


CFloatImage ColorHelper::ConvertChannels(ChannelsType dstChannelType, const CFloatImage &src)
{
	CShape dstShape = src.Shape();
	switch(dstChannelType)
	{
	case CT_RGB:
	case CT_YCrCb:
	case CT_HSV:
	case CT_LAB:
	case CT_YUV:
	case CT_LOG_RGB:
	case CT_LOG_YCrCb:
	case CT_LOG_YUV:
		dstShape.nBands = 3;
		break;
	case CT_LUM:
	case CT_LOG_LUM:
		dstShape.nBands = 1;
		break;
	default:
		REPORT_FAILURE("Unknown color channels type - %i\n", dstChannelType);
		break;
	}

	CFloatImage dst(dstShape);
	ColorHelper::ConvertChannels(dst, dstChannelType, src);

	return dst;
}

void ColorHelper::ConvertChannels(CFloatImage &dst, ChannelsType dstChannelType, CFloatImage src)
{
	CShape imgShape = src.Shape();
	INSIST((imgShape.nBands == 1) || (imgShape.nBands == 3));
	INSIST((dst.Shape().nBands == 1) || (dst.Shape().nBands == 3));
	INSIST(imgShape.SameIgnoringNBands(dst.Shape()));
	
	CFloatImage srcRGB;

	if(dstChannelType == src.channelsType)
		ImageProcessing::CopyChannels(dst, src);
	else
	{
		if(src.channelsType == CT_RGB)
			srcRGB = src;
		else
		{
			CShape rgbShape(imgShape.width, imgShape.height, 3);
			srcRGB.ReAllocate(rgbShape);
			switch(src.channelsType)
			{
			case CT_YCrCb:
				ColorHelper::YCbCrtoRGB(srcRGB, src);
				break;
			case CT_HSV:
				ColorHelper::HSVtoRGB(srcRGB, src);
				break;
			case CT_LAB:
				ColorHelper::LABtoRGB(srcRGB, src);
				break;
			case CT_YUV:
				ColorHelper::YUVtoRGB(srcRGB, src);
				break;
			case CT_LUM:
				ColorHelper::LumtoRGB(srcRGB, src);
				break;
			case CT_LOG_LUM:
				ColorHelper::LogLumtoRGB(srcRGB, src);
				break;
			case CT_LOG_RGB:
				ColorHelper::LogRgbtoRgb(srcRGB, src);
				break;
			case CT_LOG_YCrCb:
				ColorHelper::LogYCrCbtoRGB(srcRGB, src);
				break;
			case CT_LOG_YUV:
				ColorHelper::LogYUVtoRGB(srcRGB, src);
				break;
			case CT_RGB_R:
			case CT_RGB_G:
			case CT_RGB_B:
				src.channelsType = CT_LUM;
				ColorHelper::LumtoRGB(srcRGB, src);
				break;
			case CT_LOG_RGB_R:
			case CT_LOG_RGB_G:
			case CT_LOG_RGB_B:
				src.channelsType = CT_LOG_LUM;
				ColorHelper::LogLumtoRGB(srcRGB, src);
				break;

			case CT_YUV_U:
			case CT_YUV_V:
				src.channelsType = CT_LUM;
				ImageProcessing::Rescale(src, 0, 1);
				ColorHelper::LumtoRGB(srcRGB, src);
				break;

			default:
				REPORT_FAILURE("Unknown color channels type - %i\n", src.channelsType);
				break;
			}
		}

		switch(dstChannelType)
		{
		case CT_YCrCb:
			ColorHelper::RGBtoYCbCr(dst, srcRGB);
			break;
		case CT_HSV:
			ColorHelper::RGBtoHSV(dst, srcRGB);
			break;
		case CT_LAB:
			ColorHelper::RGBtoLAB(dst, srcRGB);
			break;
		case CT_YUV:
			ColorHelper::RGBtoYUV(dst, srcRGB);
			break;
		case CT_RGB:
			ImageProcessing::CopyChannels(dst, srcRGB);
			break;
		case CT_LUM:
			ColorHelper::RGBtoLum(dst, srcRGB);
			break;
		case CT_LOG_LUM:
			ColorHelper::RGBtoLogLum(dst, srcRGB);
			break;
		case CT_LOG_RGB:
			ColorHelper::RgbtoLogRgb(dst, srcRGB);
			break;
		case CT_LOG_YCrCb:
			ColorHelper::RGBtoLogYCrCb(dst, srcRGB);
			break;
		case CT_LOG_YUV:
			ColorHelper::RGBtoLogYUV(dst, srcRGB);
			break;
		default:
			REPORT_FAILURE("Unknown color channels type - %i\n", dstChannelType);
			break;
		}
	}
}

void ColorHelper::MergeChannels(CShortImage* imgChannels, CFloatImage& yCrCbImg)
{
	CShape imgShape = yCrCbImg.Shape();

	CShape channelShapes[3];
	for(int iChannel = 0; iChannel < 3; iChannel++)
	{
		channelShapes[iChannel] = imgChannels[iChannel].Shape();
	}

	int chromaSamplingX = channelShapes[0].width  / channelShapes[1].width;
	int chromaSamplingY = channelShapes[0].height / channelShapes[1].height;

	INSIST(imgShape.nBands == 3);
	INSIST(imgShape.SameIgnoringNBands(channelShapes[0]));
	INSIST(channelShapes[1] == channelShapes[2]);
	CHECK_RANGE(chromaSamplingX, 1, 2);
	CHECK_RANGE(chromaSamplingY, 1, 2);
	
	uint pixelAddr = 0;
	uint nodeAddr  = 0;
	for(int y = 0; y < channelShapes[0].height; y++)
	for(int x = 0; x < channelShapes[0].width;  x++, pixelAddr += 3, nodeAddr++)
	{
		yCrCbImg[pixelAddr + 0] = imgChannels[0][nodeAddr] / 255.0f;

		int xx = x / chromaSamplingX;
		int yy = y / chromaSamplingY;
		SET_IF_LT(xx, channelShapes[1].width  - 1);
		SET_IF_LT(yy, channelShapes[1].height - 1);
		uint chromaNodeAddr = channelShapes[1].NodeIndex(xx, yy);

		yCrCbImg[pixelAddr + 1] = imgChannels[1][chromaNodeAddr] / 255.0f;
		yCrCbImg[pixelAddr + 2] = imgChannels[2][chromaNodeAddr] / 255.0f;
	}
}

void ColorHelper::MergeChannels_YCrCb(CShortImage* imgChannels, CFloatImage& yCrCbImg)
{
	yCrCbImg.channelsType = CT_YCrCb;
	MergeChannels(imgChannels, yCrCbImg);
}

void ColorHelper::MergeChannels_YUV(CShortImage* imgChannels, CFloatImage& yuvImg)
{
	yuvImg.channelsType = CT_YUV;
	MergeChannels(imgChannels, yuvImg);
}

ChannelsType ColorHelper::GetSubChannelType(ChannelsType channelsType, int iSubChannel, bool tolerateUnknowns)
{
	CHECK_RANGE((int) channelsType, 0, (int) CT_LAST_INDEX);
	CHECK_RANGE(iSubChannel, 0, 2);

	ChannelsType subChannelType = SubChannelType[channelsType][iSubChannel];
	if(tolerateUnknowns == false)
	{
		INSIST(subChannelType != CT_UNKNOWN);
	}

	return subChannelType;
}

string ColorHelper::GetChannelsTypeStr(ChannelsType channelsType, bool tolerateUnknowns)
{
	CHECK_RANGE((int) channelsType, 0, (int) CT_LAST_INDEX);

	string ctStr = ChannelsTypeStr[channelsType];
	if(tolerateUnknowns == false)
	{
		INSIST(channelsType != CT_UNKNOWN);
	}

	return ctStr;
}


double ColorHelper::GetGradScale(ChannelsType channelsType)
{
	double scale = 1.0;

	switch(channelsType)
	{
		case CT_RGB:
		case CT_YCrCb:
		case CT_HSV:
		case CT_LAB:
		case CT_YUV:
		case CT_LUM:
			scale = 1.0;
			break;
		case CT_LOG_LUM:
		case CT_LOG_RGB:
		case CT_LOG_YCrCb:
		case CT_LOG_YUV:
		case CT_LOG_RGB_R:
		case CT_LOG_RGB_G:
		case CT_LOG_RGB_B:
		case CT_LOG_YCrCb_Cr:
		case CT_LOG_YCrCb_Cb:
		case CT_LOG_YUV_U:
		case CT_LOG_YUV_V:
			scale = 1.0 / -log(LogLumE);
			break;

		default:
			REPORT_FAILURE("Unknown channels type - %i\n", channelsType);
			break;
	};
	
	return scale;
}

