#include "rgbe-io.h"
#include "ImageIO.h"
#include "ImageProcessing.h"

void ImageIO::WriteFileRGBE(const CFloatImage& img, const char* fileName)
{
	CShape imgShape = img.Shape();
	INSIST(imgShape.nBands == 3);

	FILE* f = fopen(fileName, "wb");
	if(f == NULL)
	{
		REPORT_FAILURE("Couldn't open file for writing - %s\n", fileName);
	}

	int rbgeApiRet;

	CFloatImage imgFlip = ImageProcessing::FlipVertical(img);

	float* imgFlipBuff = (float*) imgFlip.PixelAddress(0,0,0);
	rbgeApiRet = RGBE_WriteHeader(f, imgShape.width, imgShape.height, NULL);
	INSIST(rbgeApiRet == RGBE_RETURN_SUCCESS);
	rbgeApiRet = RGBE_WritePixels(f, imgFlipBuff, imgShape.width * imgShape.height);
	INSIST(rbgeApiRet == RGBE_RETURN_SUCCESS);

	fclose(f);
}

void ImageIO::WriteFileRGBE(const CByteImage& img, const char* fileName)
{
	CFloatImage imgFlt = ImageProcessing::ByteToFloatImg(img);
	WriteFileRGBE(imgFlt, fileName);
}

void ImageIO::WriteFileRGBE(const CHalfImage& img, const char* fileName)
{
	CFloatImage imgFlt = ImageProcessing::HalfToFloatImg(img);
	WriteFileRGBE(imgFlt, fileName);
}



void ImageIO::ReadFileRGBE(CFloatImage& img, const char* fileName)
{	
	FILE* f = fopen(fileName, "rb");
	if(f == NULL)
	{
		REPORT_FAILURE("Couldn't open file for reading - %s\n", fileName);
	}

	int rbgeApiRet;
	
	CShape imgShape(0,0,3);
	rbgeApiRet = RGBE_ReadHeader(f, &imgShape.width, &imgShape.height, NULL);
	INSIST(rbgeApiRet == RGBE_RETURN_SUCCESS);
	CFloatImage imgFlip(imgShape);
	float* imgFlipBuff = (float*) imgFlip.PixelAddress(0,0,0);
	rbgeApiRet = RGBE_ReadPixels_RLE(f, imgFlipBuff, imgShape.width, imgShape.height);
	INSIST(rbgeApiRet == RGBE_RETURN_SUCCESS);

	img = ImageProcessing::FlipVertical(imgFlip);
	ImageProcessing::Rescale(img, 0.0f, 1.0f);

	fclose(f);
}

void ImageIO::ReadFileRGBE(CByteImage& img, const char* fileName)
{
	CFloatImage imgFlt;
	ReadFileRGBE(imgFlt, fileName);	
	img = ImageProcessing::FloatToByteImg(imgFlt);	
}

void ImageIO::ReadFileRGBE(CHalfImage& img, const char* fileName)
{
	CFloatImage imgFlt;
	ReadFileRGBE(imgFlt, fileName);	
	img = ImageProcessing::FloatToHalfImg(imgFlt);	
}
