template<class T>
void ImageIO::WriteRaw(const CImageOf<T> &rawImage, const char *fname)
{
	CShape imgShape = rawImage.Shape();	
	ENSURE(imgShape.depth == 1);

	uint frameSize = imgShape.width * imgShape.height * imgShape.nBands;
	T *tempBuffer  = new T[frameSize]; 
	ENSURE(tempBuffer != NULL);

	uint nodeAddr = 0;
	for (int y = 0; y < imgShape.height; y++)
	{
		for(int x = 0; x < imgShape.width; x++)
		{
			for(int b = 0; b < imgShape.nBands; b++, nodeAddr++)
			{
				tempBuffer[nodeAddr] = rawImage[nodeAddr];
			}
		}
	}

	FILE *filePtr = fopen(fname, "wb");
	if(filePtr == NULL)
	{
		REPORT_FAILURE("Could not open file for writing - %s\n", fname);
	}
	if((fwrite(&imgShape.width,        sizeof(imgShape.width),        1, filePtr) != 1) ||
	   (fwrite(&imgShape.height,       sizeof(imgShape.height),       1, filePtr) != 1) ||
	   (fwrite(&imgShape.nBands,       sizeof(imgShape.nBands),       1, filePtr) != 1) ||
	   (fwrite(&rawImage.channelsType, sizeof(rawImage.channelsType), 1, filePtr) != 1))
	{
		REPORT_FAILURE("Error writing raw file");
	}
	if(fwrite(tempBuffer, sizeof(*tempBuffer), frameSize, filePtr) != frameSize)
	{
		REPORT_FAILURE("Error writing raw file");
	}
	fclose(filePtr);

	delete [] tempBuffer;
}


template<class T>
void ImageIO::ReadRaw(CImageOf<T> &rawImage,
					  const char *fname, 
					  float scale)
{	
	FILE *filePtr = fopen(fname, "rb");
	if(filePtr == NULL)
	{
		PRINT_STR(fname);
	}
	ENSURE(filePtr != NULL);

	CShape imgShape;
	imgShape.depth  = 1;
	if((fread(&imgShape.width,        sizeof(imgShape.width),        1, filePtr) != 1) ||
	   (fread(&imgShape.height,       sizeof(imgShape.height),       1, filePtr) != 1) ||
	   (fread(&imgShape.nBands,       sizeof(imgShape.nBands),       1, filePtr) != 1) ||
	   (fread(&rawImage.channelsType, sizeof(rawImage.channelsType), 1, filePtr) != 1))
	{
		REPORT_FAILURE("Error reading raw file");
	}

	uint frameSize = imgShape.width * imgShape.height * imgShape.nBands;
	T *tempBuffer  = new T[frameSize]; 
	ENSURE(tempBuffer != NULL);

	if(fread(tempBuffer, sizeof(*tempBuffer), frameSize, filePtr) != frameSize)
	{
		REPORT_FAILURE("Error reading raw file");
	}
	fclose(filePtr);

	rawImage.ReAllocate(imgShape);
	uint nodeAddr = 0;
	for (int y = 0; y < imgShape.height; y++)
	{
		for(int x = 0; x < imgShape.width; x++)
		{
			for(int b = 0; b < imgShape.nBands; b++, nodeAddr++)
			{
				rawImage[nodeAddr] = tempBuffer[nodeAddr];
			}
		}
	}

	delete [] tempBuffer;

	if(scale != 1.0f)
	{			
		REPORT_FAILURE("Scaling raw images not implemented\n");
	}
}



template<class T>
void ImageIO::WriteFile(const CImageOf<T>& img, const char* filename)
{
    // Determine the file extension
    const char *dot = strrchr(filename, '.');
	if(dot == NULL)
	{
		REPORT_FAILURE("No file extention found - %s", filename);
	}

	string ext = (dot + 1);
	ext = tolower(ext);

	if(ext.compare("tga") == 0)
    {
		WriteFileTGA(img, filename);
    }
	else if(ext.compare("tif") == 0)
    {
		WriteFileTiff(img, filename);
	}
	else if((ext.compare("jpg")  == 0) ||
			(ext.compare("jpeg") == 0))
    {
		WriteFileJPG(img, filename);
	}
	else if((ext.compare("hdr")  == 0) ||
			(ext.compare("pic") == 0))
    {
		WriteFileRGBE(img, filename);
	}
	else if(ext.compare("raw") == 0)
	{
		WriteRaw(img, filename);
	}
    else
	{
		REPORT_FAILURE("File type not supported - %s", filename);
	}
}
