#include "VectorField.h"
#include "Disk.h"
#include "MathHelper.h"
#include "ImageProcessing.h"


VectorField::VectorField()
{
}

VectorField::VectorField(CShape shape) : CFloatImage(shape)
{
}

VectorField::VectorField(CShape shape, string dataFN) : CFloatImage(shape)
{
	LoadFieldEugene(shape, dataFN);
}


void VectorField::LoadFieldEugene(CShape shape, string dataFN)
{
	INSIST(Disk::DoesFileExist(dataFN) == true);
	INSIST(shape.nBands == 2);

	this->ReAllocate(shape);
	FILE* filePtr = fopen(dataFN.c_str(), "r");
		
	for(int y = 0; y < shape.height; y++)
	{
		uint pixelAddr = shape.PixelIndex(0, shape.height - 1 - y, 0);
		for(int x = 0; x < shape.width;  x++, pixelAddr += shape.nBands)
		{
			INSIST(feof(filePtr) == 0);
			float pixelX, pixelY, pixelU, pixelV, pixelVecX, pixelVecY;

			fscanf(filePtr, "%f %f %f %f %f %f\n", 
					&pixelY,    &pixelX,
					&pixelU,    &pixelV,
					&pixelVecX, &pixelVecY);

			INSIST(pixelX == x);
			INSIST(pixelY == y);

			(*this)[pixelAddr + 0] = pixelVecX;
			(*this)[pixelAddr + 1] = pixelVecY;
		}	
	}

	fclose(filePtr);	
}

void VectorField::NormalizeField()
{
	CShape shape = this->Shape();

	uint pixelAddr = 0;
	for(float y = 0; y < shape.height; y++)
	for(float x = 0; x < shape.width;  x++, pixelAddr += shape.nBands)
	{
		float vecX = (*this)[pixelAddr + 0];
		float vecY = (*this)[pixelAddr + 1];

		float vecLen = sqrt((vecX * vecX) + (vecY * vecY));
		if(vecLen != 0.0)
		{
			vecX /= vecLen;
			vecY /= vecLen;
		}

		(*this)[pixelAddr + 0] = vecX;
		(*this)[pixelAddr + 1] = vecY;
	}
}

void VectorField::InitDiagonalField(float angDegrees)
{
	CShape shape = this->Shape();

	float angRadians = MathHelper::DegressToRadians(angDegrees);
	float vecX = cos(angRadians);
	float vecY = sin(angRadians);

	uint pixelAddr = 0;
	for(int y = 0; y < shape.height; y++)
	for(int x = 0; x < shape.width;  x++, pixelAddr += shape.nBands)
	{
		(*this)[pixelAddr + 0] = vecX;
		(*this)[pixelAddr + 1] = vecY;
	}

	NormalizeField();
}


void VectorField::InitSinkField(float sinkX, float sinkY, bool invertY)
{
	CShape shape = this->Shape();
	
	if(invertY == true)
	{
		CHECK_RANGE(sinkY, 0, shape.height - 1);
		sinkY = shape.height - 1 - sinkY;
	}

	uint pixelAddr = 0;
	for(float y = 0; y < shape.height; y++)
	for(float x = 0; x < shape.width;  x++, pixelAddr += shape.nBands)
	{
		(*this)[pixelAddr + 0] = sinkX - x;
		(*this)[pixelAddr + 1] = sinkY - y;
	}

	NormalizeField();
}


void VectorField::InitSourceField(float sourceX, float sourceY, bool invertY)
{
	CShape shape = this->Shape();

	if(invertY == true)
	{
		CHECK_RANGE(sourceY, 0, shape.height - 1);
		sourceY = shape.height - 1 - sourceY;
	}

	uint pixelAddr = 0;
	for(float y = 0; y < shape.height; y++)
	for(float x = 0; x < shape.width;  x++, pixelAddr += shape.nBands)
	{
		(*this)[pixelAddr + 0] = x - sourceX;
		(*this)[pixelAddr + 1] = y - sourceY;
	}

	NormalizeField();
}
