#include "GS_FilterWindow.h"
#include "GS_MainWindow.h"
#include "GS_Curve_Panel.h"

void
GS_FilterWindow::setup_Widgets()
{
	this->begin();

	int widgetLocX = 20, widgetLocY = 20;

	setupSolverWidgets(widgetLocX, widgetLocY);

	widgetLocY += 40;

	switch(this->fltParams->type)
	{
	case Filter_Driver::FT_NPR:
		setupFltWidgets_npr(widgetLocX, widgetLocY);
		break;

	case Filter_Driver::FT_DEBLOCK:
		setupFltWidgets_deblock(widgetLocX, widgetLocY);
		break;

	case Filter_Driver::FT_TM:
		setupFltWidgets_tm(widgetLocX, widgetLocY);
		break;

	default:
		REPORT_FAILURE("Unknown filter type - %i\n", this->fltParams->type);
		break;
	}

	this->end();
}

void 
GS_FilterWindow::setupSolverWidgets(int& locX, int& locY)
{
	int widgetX, widgetW, widgetH;
	widgetH = 30;

	widgetW = 200;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_gss_fideltyD = new Fl_Hor_Value_Slider(widgetX, locY, widgetW, widgetH, "Lambda-D");
	this->wg_gss_fideltyD->when(FL_WHEN_CHANGED);
	this->wg_gss_fideltyD->callback(&GS_FilterWindow::lamdaD_changed, this);
	float sliderVal = lambdaDToSlider((float) this->fltParams->gssParams.fidelity_data);
	this->wg_gss_fideltyD->value(sliderVal);
	locY += widgetH;

	locY += 30;

	widgetW = 100;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_solve = new Fl_Button(widgetX, locY, widgetW, widgetH, "Solve");
	this->wg_solve->callback(&GS_FilterWindow::solve, this);
	locY += widgetH;
}


void
GS_FilterWindow::setupFltWidgets_npr(int& locX, int& locY)
{
	int widgetX, widgetW, widgetH;
	widgetH = 30;

	widgetW = 200;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_gss_eleNorm = new Fl_Hor_Value_Slider(widgetX, locY, widgetW, widgetH, "Edge length norm");
	this->wg_gss_eleNorm->when(FL_WHEN_CHANGED);
	this->wg_gss_eleNorm->callback(&GS_FilterWindow::elNorm_changed, this);
	float sliderVal = elNornToSlider((float) this->fltParams->gsf_nprParams.edgeLenNorm);
	this->wg_gss_eleNorm->value(sliderVal);
	locY += widgetH;

	locY += 30;

	GS_Curve_Panel::GS_Curve_Panel_Params cpParams = GS_Curve_Panel::GS_Curve_Panel_Params::Default();
	cpParams.width = 300;
	cpParams.height = 300;
	widgetW = cpParams.width;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_curveEditor = new GS_Curve_Panel(&this->fltParams->gsf_nprParams.edgeLenCurve, cpParams);
	this->wg_curveEditor->position(widgetX, locY);
	this->wg_curveEditor->size(cpParams.width, cpParams.height);
	locY += cpParams.height;

	locY += 30;

	widgetW = 100;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_filterAndSolve = new Fl_Button(widgetX, locY, widgetW, widgetH, "Filter N Solve");
	this->wg_filterAndSolve->callback(&GS_FilterWindow::filterAndSolve, this);
	locY += widgetH;
}


void
GS_FilterWindow::setupFltWidgets_tm(int& locX, int& locY)
{
	int widgetX, widgetW, widgetH;
	widgetH = 30;

	widgetW = 200;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_tm_scale = new Fl_Hor_Value_Slider(widgetX, locY, widgetW, widgetH, "tone mapping scale");
	this->wg_tm_scale->when(FL_WHEN_CHANGED);
	this->wg_tm_scale->callback(&GS_FilterWindow::tm_scale_changed, this);
	this->wg_tm_scale->value(this->fltParams->gsf_tmParams.attenFuncScale);
	locY += widgetH;

	locY += 30;


	widgetW = 200;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_tm_exp = new Fl_Hor_Value_Slider(widgetX, locY, widgetW, widgetH, "tone mapping exp");
	this->wg_tm_exp->when(FL_WHEN_CHANGED);
	this->wg_tm_exp->callback(&GS_FilterWindow::tm_exp_changed, this);
	this->wg_tm_exp->value(this->fltParams->gsf_tmParams.attenFuncExp);
	locY += widgetH;

	locY += 30;


	widgetW = 100;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_filterAndSolve = new Fl_Button(widgetX, locY, widgetW, widgetH, "Filter N Solve");
	this->wg_filterAndSolve->callback(&GS_FilterWindow::filterAndSolve, this);
	locY += widgetH;
}


void
GS_FilterWindow::setupFltWidgets_deblock(int& locX, int& locY)
{
	int widgetX, widgetW, widgetH;
	widgetH = 30;

	GS_Curve_Panel::GS_Curve_Panel_Params cpParams = GS_Curve_Panel::GS_Curve_Panel_Params::Default();
	cpParams.width = 300;
	cpParams.height = 300;
	widgetW = cpParams.width;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_curveEditor = new GS_Curve_Panel(&this->fltParams->gsf_deblockParams.blockGradCurve, cpParams);
	this->wg_curveEditor->position(widgetX, locY);
	this->wg_curveEditor->size(cpParams.width, cpParams.height);
	locY += cpParams.height;

	locY += 30;

	widgetW = 100;
	widgetX = (this->w() - (2 * locX) - widgetW) / 2 + locX;
	this->wg_filterAndSolve = new Fl_Button(widgetX, locY, widgetW, widgetH, "Filter N Solve");
	this->wg_filterAndSolve->callback(&GS_FilterWindow::filterAndSolve, this);
	locY += widgetH;
}

void 
GS_FilterWindow::solve(Fl_Widget *, void* obj)
{
	GS_FilterWindow* fltWin = (GS_FilterWindow*) obj;
	GS_MainWindow* mainWin  = (GS_MainWindow*) fltWin->parentWin;

	mainWin->executeSolver();
}

void 
GS_FilterWindow::filterAndSolve(Fl_Widget *, void* obj)
{
	GS_FilterWindow* fltWin = (GS_FilterWindow*) obj;
	GS_MainWindow* mainWin  = (GS_MainWindow*) fltWin->parentWin;

	mainWin->executeFilter(fltWin->fltParams->type);
}

void 
GS_FilterWindow::lamdaD_changed(Fl_Widget *, void* obj)
{
	GS_FilterWindow* fltWin = (GS_FilterWindow*) obj;
	fltWin->fltParams->gssParams.fidelity_data = sliderToLamdaD((float) fltWin->wg_gss_fideltyD->value());
}

void 
GS_FilterWindow::elNorm_changed(Fl_Widget *, void* obj)
{
	GS_FilterWindow* fltWin = (GS_FilterWindow*) obj;
	fltWin->fltParams->gsf_nprParams.edgeLenNorm = sliderToElNorm((float) fltWin->wg_gss_eleNorm->value());
}


void 
GS_FilterWindow::tm_exp_changed(Fl_Widget *, void* obj)
{
	GS_FilterWindow* fltWin = (GS_FilterWindow*) obj;
	fltWin->fltParams->gsf_tmParams.attenFuncExp = (float) fltWin->wg_tm_exp->value();
}


void 
GS_FilterWindow::tm_scale_changed(Fl_Widget *, void* obj)
{
	GS_FilterWindow* fltWin = (GS_FilterWindow*) obj;
	fltWin->fltParams->gsf_tmParams.attenFuncScale = (float) fltWin->wg_tm_scale->value();
}

float 
GS_FilterWindow::sliderToLamdaD(float sliderVal)
{
	CHECK_RANGE(sliderVal, 0.0f, 1.0f);
	float lambdaD;
	if(sliderVal == 0.0)
		lambdaD = 0.0f;
	else
		lambdaD =  pow(10.0f, (sliderVal * 7.0f - 7.0f));

	return lambdaD;
}

float 
GS_FilterWindow::lambdaDToSlider(float lambdaD)
{
	CHECK_RANGE(lambdaD, 0.0f, 1.0f);

	float sliderVal;
	if(lambdaD == 0.0f)
		sliderVal = 0.0f;
	else
		sliderVal = sliderVal = (7.0f + log10(lambdaD)) / 7.0f;

	SET_IF_GT(sliderVal, 0.0f);
	SET_IF_LT(sliderVal, 1.0f);

	return sliderVal;
}


float 
GS_FilterWindow::sliderToElNorm(float sliderVal)
{
	CHECK_RANGE(sliderVal, 0.0f, 1.0f);
	float elNorm;
	elNorm = sliderVal * (5000.0f - 250.0f) + 250.0f;;
	return elNorm;
}

float 
GS_FilterWindow::elNornToSlider(float elNorm)
{
	CHECK_RANGE(elNorm, 250.0f, 5000.0f);
	float sliderVal = (elNorm - 250.0f) / (5000.0f - 250.0f);
	SET_IF_GT(sliderVal, 0.0f);
	SET_IF_LT(sliderVal, 1.0f);
	return sliderVal;
}
