#pragma once

#include "Definitions.h"
#include "GS_UI_Definitions.h"
#include "ImageData.h"
#include "GS_Curve_Panel.h"
#include "OptFileParser.h"

class GS_FilterWindow : public Fl_Double_Window
{
public:
	typedef struct _Params
	{
		int width, height;
		string title;
		static _Params Default()
		{
			_Params params;
			params.width  = 400;
			params.height = 600;
			params.title  = ":: Painterly filter ::";
			return params;
		}
	} Params;

private:
	Fl_Hor_Value_Slider* wg_gss_fideltyD;
	Fl_Hor_Value_Slider* wg_gss_eleNorm;

	Fl_Hor_Value_Slider* wg_tm_scale;
	Fl_Hor_Value_Slider* wg_tm_exp;

	GS_Curve_Panel*      wg_curveEditor;
	Fl_Button*           wg_solve;
	Fl_Button*           wg_filterAndSolve;

	void* parentWin;
	Filter_Driver::Params* fltParams;

public:
	GS_FilterWindow(void* mainWin, Filter_Driver::Params* _fltParams, Params _params = Params::Default());

private:
	void setup_VarDefaults();

private: //GS_FilterWindow-wg.cpp
	void setup_Widgets();
	void setupSolverWidgets(int& locX, int& locY);
	void setupFltWidgets_npr(int& locX, int& locY);
	void setupFltWidgets_deblock(int& locX, int& locY);
	void setupFltWidgets_tm(int& locX, int& locY);

	static void solve(Fl_Widget *, void*);
	static void filterAndSolve(Fl_Widget *, void*);

	static void lamdaD_changed(Fl_Widget *, void*);
	static void elNorm_changed(Fl_Widget *, void*);
	static void tm_scale_changed(Fl_Widget *, void*);
	static void tm_exp_changed(Fl_Widget *, void*);

	static float sliderToLamdaD(float sliderVal);
	static float lambdaDToSlider(float lambdaD);

	static float sliderToElNorm(float sliderVal);
	static float elNornToSlider(float elNorm);

public: //GS_FilterWindow-Events.cpp
	int handle(int eventID);
};

