#include "GS_ImagePanel.h"
#include "FL/Fl_JPEG_Image.H"

GS_ImagePanel::GS_ImagePanel(ImageData *imageData) : Fl_Box(0, 0, 0, 0, NULL)
{
	INSIST(imageData != NULL);

	this->imgData = imageData;
	this->wg_displayImg  = NULL;
	this->displayImgData = NULL;
	this->panelType      = PT_DEFAULT;

	Callback imgContentUpdate(this, GS_ImagePanel::UpdateImgContent);
	this->imgData->Add_contentUpdate_callback(imgContentUpdate);
}

GS_ImagePanel::~GS_ImagePanel()
{
	delete this->wg_displayImg;
	delete [] this->displayImgData;
}

void 
GS_ImagePanel::SetPanelType(PanelType _panelType)
{
	this->panelType = _panelType;
	if(this->imgData->imgShape.NodeCount() > 0)
		updateDisplay();
}

GS_ImagePanel::PanelType
GS_ImagePanel::GetPanelType()
{
	return this->panelType;
}

void GS_ImagePanel::Add_resize_callback(Callback callback)
{
	this->resizeCallbacks.push_back(callback);
}

void GS_ImagePanel::make_resize_callbacks()
{
	for(uint iCallback = 0; iCallback < this->resizeCallbacks.size(); iCallback++)
	{
		Callback callback = this->resizeCallbacks[iCallback];
		callback.second(callback.first);
	}
}

void GS_ImagePanel::UpdateImgContent(void *imgPanelPtr)
{
	GS_ImagePanel *imgPanel = (GS_ImagePanel *) imgPanelPtr;
	imgPanel->updateDisplay();
}

void GS_ImagePanel::updateDisplay()
{
	CShape imgShape = this->imgData->imgShape;
	INSIST((imgShape.nBands == 1) || (imgShape.nBands == 3));

	if((this->displayImgData == NULL)                 ||
	   (this->wg_displayImg->w() != imgShape.width)  ||
	   (this->wg_displayImg->h() != imgShape.height) ||
	   (this->wg_displayImg->d() != imgShape.nBands))
	{
		delete [] this->displayImgData;
		this->displayImgData = new uchar[imgShape.width * imgShape.height * imgShape.nBands];
		this->size(imgShape.width, imgShape.height);
		make_resize_callbacks();
	}

	CByteImage panelSrc = getPanelSourceImg();
	INSIST(panelSrc.Shape() == imgShape);

	uint pixelAddr = 0;
	for(int y = 0; y < imgShape.height;  y++)
	{
		uint invertedPixelAddr = imgShape.PixelIndex(0, imgShape.height - 1 - y, 0);
		for(int x = 0; x < imgShape.width;   x++)
		for(int c = 0; c < imgShape.nBands;  c++, pixelAddr++, invertedPixelAddr++)
		{
			this->displayImgData[pixelAddr] = panelSrc[invertedPixelAddr];
		}
	}	

	delete this->wg_displayImg;
	this->wg_displayImg  = new Fl_RGB_Image(this->displayImgData,
									        imgShape.width, imgShape.height, imgShape.nBands);
	this->image(this->wg_displayImg);
	this->redraw();
}


CByteImage
GS_ImagePanel::getPanelSourceImg()
{
	CByteImage srcImg;

	switch(this->panelType)
	{
	case PT_IMG:
		srcImg = this->imgData->img;
		break;

	case PT_RESULT:
		srcImg = this->imgData->result;
		break;

	default:
		REPORT_FAILURE("Unknown panel type - %i\n", this->panelType);
		break;
	}

	return srcImg;
}

