#include "ImageData.h"
#include "ImageIO.h"

ImageData::ImageData()
{
	this->imgShape = CShape(0, 0, 0, 0);	
	this->imgFileName = "";
	this->prob = NULL;
}

void ImageData::Load(string fileName, GS_Problem::Params probParams)
{
	INCR_TAB_LEVEL();
	REPORT("Loading image: %s\n", fileName.c_str());
	
	this->imgFileName = fileName;
	ImageIO::ReadFile(this->img, this->imgFileName);
	this->imgShape = this->img.Shape();
	this->result.ReAllocate(this->imgShape);

	GS_ProblemInput::Params inputParams = GS_ProblemInput::Params::Default();
	inputParams.loadOnlyIfExists = true;
	inputParams.loadEmData = true;
	GS_ProblemInput probInput(fileName, inputParams, probParams.channelsType);

	delete this->prob;
	this->prob = new GS_Problem(probInput, probParams);
	INSIST(this->prob != NULL);

	Callback callback(this, &ImageData::ResultUpdateCallback); 
	this->prob->Add_resultUpdate_callback(callback);
	getCurrentResult();

	make_contentUpdate_callbacks();

	REPORT("Image dimensions - [%i, %i, %i]\n", 
			this->imgShape.width, this->imgShape.height, this->imgShape.nBands);
	DCR_TAB_LEVEL();
}


void ImageData::Add_contentUpdate_callback(Callback callback)
{
	this->contentUpdate_Callbacks.push_back(callback);
}

void ImageData::make_contentUpdate_callbacks()
{
	for(uint iCallback = 0; iCallback < this->contentUpdate_Callbacks.size(); iCallback++)
	{
		Callback callback = this->contentUpdate_Callbacks[iCallback];
		callback.second(callback.first);
	}
}


void 
ImageData::ResultUpdateCallback(void* objPtr)
{
	ImageData* imgDataPtr = (ImageData*) objPtr;
	imgDataPtr->resultUpdateCallback();
}


void 
ImageData::resultUpdateCallback()
{
	getCurrentResult();
	make_contentUpdate_callbacks();
}


void 
ImageData::getCurrentResult()
{
	GS_Image resultFlt = this->prob->GetResultRGB(true);
	this->result = ImageProcessing::FloatToByteImg(resultFlt);
}
