#include "GS_Curve_Panel.h"

int GS_Curve_Panel::handle(int eventID)
{
	bool handled = false;	

	int clickX = Fl::event_x();
	int clickY = Fl::event_y();

	switch(eventID) 
	{
	case FL_PUSH:
		handled = handle_push(clickX, clickY);		
		break;

	case FL_SHORTCUT:
		handled = handle_key_press();
		break;

	case FL_DRAG:
		handled = handle_drag(clickX, clickY);
		break;

	case FL_MOVE:
		if(this->selectedPtIndex == -1)
		{
			updateInOutInputs();
		}
		break;
	}

	if(handled == false)
//		return Fl_Double_Window::handle(eventID);
		return Fl_Group::handle(eventID);
	else
	{
		paramUpdate();
		return 1;
	}	
}

bool GS_Curve_Panel::handle_push(int clickX, int clickY)
{
	bool handled = false;

	if(Fl::event_button() == FL_LEFT_MOUSE)
	{
		double graphX, graphY;
		if(isPointInGraph(clickX, clickY, graphX, graphY))
		{
			this->selectedPtIndex = this->curvePoints->GetNearestPointIndex(graphX, graphY, 
																		    GS_CURVE_PANEL_PtSelectionDistThres);
			if(this->selectedPtIndex == -1)
			{
				CurvePoint newCurvePoint(graphX, graphY);
				this->selectedPtIndex = this->curvePoints->insert(newCurvePoint);
			}

			this->wg_Curve_Vis->take_focus();

			handled = true;
		}
	}

	return handled;
}

bool GS_Curve_Panel::handle_release(int clickX, int clickY)
{
	bool handled = false;

	if(Fl::event_button() == FL_LEFT_MOUSE)		
	{
		release_selection();		
		handled = true;
	}

	return handled;
}

void GS_Curve_Panel::release_selection()
{
	this->selectedPtIndex = -1;
}

bool GS_Curve_Panel::handle_drag(int clickX, int clickY)
{
	bool handled = false;

	if((this->selectedPtIndex != -1) &&
	   (Fl::event_button() == FL_LEFT_MOUSE))
	{
		double graphX, graphY;
		if(isPointInGraph(clickX, clickY, graphX, graphY))
		{
			double newPtX = graphX;
			double newPtY = graphY;
			if(this->selectedPtIndex > 0)
			{
				newPtX = max(this->curvePoints->at(this->selectedPtIndex - 1).x, newPtX);
			}

			if(this->selectedPtIndex < (int) this->curvePoints->size() - 1)
			{
				newPtX = min(this->curvePoints->at(this->selectedPtIndex + 1).x, newPtX);
			}

			CurvePoint modifiedPt(newPtX, newPtY);
			this->curvePoints->modify(this->selectedPtIndex, modifiedPt);
			handled = true;
		}
	}

	return handled;
}


bool GS_Curve_Panel::handle_key_press()
{
	bool handled = false;

	if((this->selectedPtIndex != -1) &&
	   (Fl::focus() == this->wg_Curve_Vis))
	{
		switch(Fl::event_key())
		{
		case FL_Delete:
			handled = handle_delete_selectedPt();
			break;

		case 'D':
		case 'd':
			handle_shift_SelectedPt(GS_CURVE_PANEL_PtShiftAmount, 0.0);
			handled = true;
			break;

		case 'A':
		case 'a':
			handle_shift_SelectedPt(-GS_CURVE_PANEL_PtShiftAmount, 0.0);
			handled = true;
			break;

		case 'W':
		case 'w':
			handle_shift_SelectedPt(0.0, GS_CURVE_PANEL_PtShiftAmount);
			handled = true;
			break;

		case 'S':
		case 's':
			handle_shift_SelectedPt(0.0, -GS_CURVE_PANEL_PtShiftAmount);
			handled = true;
			break;
		}
	}

	return handled;
}

bool GS_Curve_Panel::handle_delete_selectedPt()
{
	bool handled = false;

	if(this->curvePoints->size() > 2)
	{
		int ptIndex = this->selectedPtIndex;
		release_selection();
		this->curvePoints->erase(ptIndex);
		handled = true;
	}

	return handled;
}

void GS_Curve_Panel::handle_shift_SelectedPt(double xShift, double yShift)
{
	INSIST(this->selectedPtIndex != -1);

	CurvePoint selectedPt = this->curvePoints->at(this->selectedPtIndex);

	selectedPt.x += xShift;
	selectedPt.y += yShift;

	selectedPt.x = max(min(selectedPt.x, 1.0), 0.0);
	selectedPt.y = max(min(selectedPt.y, 1.0), 0.0);
	
	if(this->selectedPtIndex > 0)
	{
		selectedPt.x = max(this->curvePoints->at(this->selectedPtIndex - 1).x, selectedPt.x);
	}

	if(this->selectedPtIndex < (int) this->curvePoints->size() - 1)
	{
		selectedPt.x = min(this->curvePoints->at(this->selectedPtIndex + 1).x, selectedPt.x);
	}
	
	this->curvePoints->modify(this->selectedPtIndex, selectedPt);
}


bool GS_Curve_Panel::isPointInGraph(int clickX, int clickY, 
									double &graphX, double &graphY)
{
	bool pointInGraph = false;

	if(IsPointInWidget(this->wg_Curve_Vis, clickX, clickY) == true)
	{
		GS_Line graphDiag = this->wg_Curve_Vis->GetGraphDiag();
		if(InRange(clickX, graphDiag.x1, graphDiag.x2) &&
		   InRange(clickY, graphDiag.y2, graphDiag.y1))
		{
			pointInGraph = true;

			int graphSpanX = graphDiag.x2 - graphDiag.x1;
			int graphSpanY = graphDiag.y1 - graphDiag.y2;
			graphX = ((double) clickX - graphDiag.x1) / graphSpanX;
			graphY = ((double) graphDiag.y1 - clickY) / graphSpanY;
			CHECK_RANGE(graphX, 0.0, 1.0);
			CHECK_RANGE(graphY, 0.0, 1.0);
		}
	}

	return pointInGraph;
}
