#include "GS_Curve_Panel.h"

void GS_Curve_Panel::MinResponseChanged(Fl_Widget *, void *GS_Curve_Panel_ptr)
{
	GS_Curve_Panel *scurve_Panel = (GS_Curve_Panel *) GS_Curve_Panel_ptr;
	Fl_Float_Input *minR = scurve_Panel->wg_minR_input;
	double newMin = atof(minR->value());
	if( (strlen(minR->value()) != 0) &&
		(newMin >= 0.0) &&
	    (newMin < scurve_Panel->curvePoints->MaxOut()) )
	{
		scurve_Panel->curvePoints->MinOut(newMin);
		scurve_Panel->paramUpdate();
	}
	else
	{
		REPORT("Invalid input. Reseting min-out to previous valid value.");
		scurve_Panel->updateResponseInputs();
	}
}

void GS_Curve_Panel::MaxResponseChanged(Fl_Widget *, void *GS_Curve_Panel_ptr)
{
	GS_Curve_Panel *scurve_Panel = (GS_Curve_Panel *) GS_Curve_Panel_ptr;
	Fl_Float_Input *maxR = scurve_Panel->wg_maxR_input;
	double newMax = atof(maxR->value());
	if( (strlen(maxR->value()) != 0) &&
		(newMax >= 0.0)  &&
	    (newMax > scurve_Panel->curvePoints->MinOut()) )
	{
		scurve_Panel->curvePoints->MaxOut(newMax);
		scurve_Panel->paramUpdate();
	}
	else
	{
		REPORT("Invalid input. Reseting max-out to previous valid value.");
		scurve_Panel->updateResponseInputs();
	}
}


void GS_Curve_Panel::InValChanged(Fl_Widget *, void *GS_Curve_Panel_ptr)
{
	GS_Curve_Panel *scurve_Panel = (GS_Curve_Panel *) GS_Curve_Panel_ptr;
	scurve_Panel->inValChanged();	
}

void GS_Curve_Panel::inValChanged()
{
	if(this->selectedPtIndex == -1) return;

	CurvePoint selectedPt = this->curvePoints->at(this->selectedPtIndex);

	double minIn = 0.0;
	double maxIn = 1.0;
	if(this->selectedPtIndex > 0)
	{
		minIn = this->curvePoints->at(this->selectedPtIndex - 1).x;
	}

	if(this->selectedPtIndex < this->curvePoints->size() - 1)
	{
		maxIn = this->curvePoints->at(this->selectedPtIndex + 1).x;
	}

	Fl_Float_Input *in = this->wg_in_input;
	double newIn = atof(in->value());
	if( (strlen(in->value()) != 0) &&
		(newIn >= minIn) &&
	    (newIn <= maxIn) )
	{		
		selectedPt.x = newIn;
		this->curvePoints->modify(this->selectedPtIndex, selectedPt);
	}

	paramUpdate();
}

void GS_Curve_Panel::OutValChanged(Fl_Widget *, void *GS_Curve_Panel_ptr)
{
	GS_Curve_Panel *scurve_Panel = (GS_Curve_Panel *) GS_Curve_Panel_ptr;
	scurve_Panel->outValChanged();
}

void GS_Curve_Panel::outValChanged()
{
	if(this->selectedPtIndex == -1) return;

	CurvePoint selectedPt = this->curvePoints->at(this->selectedPtIndex);

	double minOut = this->curvePoints->MinOut();
	double maxOut = this->curvePoints->MaxOut();

	Fl_Float_Input *out = this->wg_out_input;
	double newOut = atof(out->value());
	if( (strlen(out->value()) != 0) &&
		(newOut >= minOut) &&
	    (newOut <= maxOut) )
	{		
		selectedPt.y = (newOut - minOut) / (maxOut - minOut);
		this->curvePoints->modify(this->selectedPtIndex, selectedPt);
	}

	paramUpdate();
}


void GS_Curve_Panel::updateInOutInputs()
{	
	double curveX = 0.0;
	
	if(this->selectedPtIndex != -1)
	{
		curveX = this->curvePoints->at(this->selectedPtIndex).x;
		this->wg_in_input->activate();
		this->wg_out_input->activate();
	}
	else
	{
		this->wg_in_input->deactivate();
		this->wg_out_input->deactivate();
		int clickX = Fl::event_x();
		int clickY = Fl::event_y();
		double graphX, graphY;
		if(isPointInGraph(clickX, clickY, graphX, graphY))
		{
			curveX = graphX;
		}
	}
	
	double curveY = this->curvePoints->GetCurveResponse(curveX);

	char valBuf[100];
	sprintf(valBuf, "%.04lf", curveX);
	this->wg_in_input->value(valBuf);
	sprintf(valBuf, "%.04lf", curveY);
	this->wg_out_input->value(valBuf);
}


void GS_Curve_Panel::updateResponseInputs()
{
	char valBuf[100];
	sprintf(valBuf, "%.01lf", this->curvePoints->MinOut());
	this->wg_minR_input->value(valBuf);
	sprintf(valBuf, "%.01lf", this->curvePoints->MaxOut());
	this->wg_maxR_input->value(valBuf);
}