#include "GS_Curve_Vis.h"
#include <FL/fl_draw.H>

GS_Curve_Vis::GS_Curve_Vis(const Curve *_curve) : Fl_Button(0, 0, 0, 0, "")
{
	this->box(FL_FLAT_BOX);
	this->color(FL_WHITE);

	this->curve = _curve;
	this->selectedPtIndex = -1;

	Callback_Double funcEvaluator(this->curve, &Curve::GetCurveResponse);
	funcVisParams.funcEvaluator = funcEvaluator;
}

void GS_Curve_Vis::UpdateParams(int _selectedPtIndex)
{
	this->selectedPtIndex = _selectedPtIndex;
	redraw();
}

void GS_Curve_Vis::updateFuncVisParams()
{
	funcVisParams.x    = this->x();
	funcVisParams.y    = this->y();
	funcVisParams.w    = this->w();
	funcVisParams.h    = this->h();
	funcVisParams.minX = 0.0;
	funcVisParams.maxX = 1.0;
	funcVisParams.minY = this->curve->MinOut();
	funcVisParams.maxY = this->curve->MaxOut();
}

GS_Line GS_Curve_Vis::GetGraphDiag()
{
	updateFuncVisParams();
	GS_Line graphDiag = GS_FuncVis::GetGraphDiag(this->funcVisParams);
	return graphDiag;
}

void GS_Curve_Vis::draw() 
{
	Fl_Button::draw();

	updateFuncVisParams();
	GS_Line graphDiag = GS_FuncVis::draw(this->funcVisParams);

	drawCurvePoints(graphDiag);
}

void GS_Curve_Vis::drawCurvePoints(const GS_Line &graphDiag)
{
	int graphSpanX = graphDiag.x2 - graphDiag.x1;
	int graphSpanY = graphDiag.y1 - graphDiag.y2;

	const int pointSize = 3;

	fl_push_clip(graphDiag.x1, graphDiag.y2 - pointSize, 
				 graphSpanX + pointSize, graphSpanY + pointSize);

	for(int iPoint = 0; iPoint < this->curve->size(); iPoint++)
	{
		CurvePoint currPoint = this->curve->at(iPoint);

		double pointDrawX = graphDiag.x1 + (currPoint.x * graphSpanX);
		double pointDrawY = graphDiag.y1 - (currPoint.y * graphSpanY);

		Fl_Color drawColor;
		if(iPoint == this->selectedPtIndex)
			drawColor = FL_RED;
		else
			drawColor = FL_GREEN;

		fl_rectf((int) pointDrawX - pointSize, (int) pointDrawY - pointSize, 
				 pointSize * 2, pointSize * 2,
				 drawColor);
	}

	fl_pop_clip();
}

int GS_Curve_Vis::handle(int eventID)
{
	switch(eventID)
	{
	case FL_PUSH:
		return 0; //avoids the button down animation
		break;
	}

	return Fl_Button::handle(eventID);
}
